@@ -10,13 +10,14 @@
 #undef CVS_SOURCE 
 #undef CORBA_INTERFACE
 #undef MOZILLA_VERSION
+#undef MOZILLA_SNAPSHOT
+#undef NEED_MOZILLA_CONFIG
 #undef ENABLE_APPLET
 #undef HAVE__NL_PAPER_WIDTH_HEIGHT
 #undef ENABLE_GNOME_FILE_SELECTOR
 #undef ENABLE_EXPERIMENTAL_FILTERS
 #undef HAVE_XINERAMA
 #undef XPCOM_SHARED_LIB
-#undef GCC_VERSION
 #undef ENABLE_NAUTILUS_VIEW
 #undef STRING_HEADER
 #undef GNOME_PIXMAPS_DIR
