@@ -1622,8 +1622,7 @@ bookmarks_folder_open_all_items (GaleonEmbed *embed, BookmarkItem *bi,
 		{
 			embed_create_after_embed (embed, new_window,
 						  child->url,
-						  EMBED_CREATE_NEVER_JUMP |
-						  EMBED_CREATE_RAISE_WINDOW);
+						  EMBED_CREATE_NEVER_JUMP);
 		}
 		l = l->next;
 	}
@@ -1764,6 +1763,9 @@ bookmarks_merge_trees (BookmarkItem *root1, BookmarkItem *root2)
 	GList *li;
 	gboolean last_one_added = FALSE;
 
+	g_return_if_fail (root1 != NULL);
+	g_return_if_fail (root2 != NULL);
+	
 	if (!BOOKMARK_ITEM_IS_FOLDER (root2) && !(root2->alias_of))
 	{
 		BookmarkItem *item =
@@ -1814,6 +1816,10 @@ bookmarks_merge_trees_look_for_equal (BookmarkItem *folder,
 				      BookmarkItem *bookmark)
 {
 	GList *li;
+
+	g_return_val_if_fail (folder != NULL, NULL);
+	g_return_val_if_fail (bookmark != NULL, NULL);
+
 	if (bookmark->type == BM_SEPARATOR) return NULL;
 	for (li = folder->list; li != NULL; li = li->next) 
 	{
@@ -1933,8 +1939,7 @@ bookmarks_folder_open_in_myportal (GaleonEmbed *embed, BookmarkItem *bi,
 	else 
 	{
 		embed_create_after_embed (embed, new_window, url,
-					  EMBED_CREATE_FORCE_JUMP |
-					  EMBED_CREATE_RAISE_WINDOW);
+					  EMBED_CREATE_FORCE_JUMP);
 	}
 	g_free (url);
 }
