@@ -15,9 +15,10 @@
 
 #include "galeon.h"
 #include "window.h"
-#include "misc.h"
 #include "dialog.h"
+#include "misc_string.h"
 #include "prefs.h"
+#include "glade.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -31,6 +32,7 @@
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-triggers.h>
+#include <libgnomeui/gnome-dialog.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-winhints.h>
 #include <libgnomeui/gnome-app.h>
@@ -46,9 +48,20 @@ static gboolean misc_ask_question_ret = FALSE;
 gboolean 
 dialog_ask_question (const gchar *question, GtkWidget *parent)
 {
-	GtkWidget *dialog = gnome_question_dialog_parented
-		(question, dialog_ask_question_cb, NULL, GTK_WINDOW (parent));
-	dialog_set_parent (dialog, parent);
+	GtkWidget *dialog;
+	if (parent)
+	{
+		dialog = gnome_question_dialog_parented
+			(question, dialog_ask_question_cb, NULL,
+			 GTK_WINDOW (parent));
+		dialog_set_parent (dialog, parent);
+	}
+	else
+	{
+		dialog = gnome_question_dialog (question,
+						dialog_ask_question_cb, NULL);
+	}
+
 	gnome_dialog_run (GNOME_DIALOG (dialog));
 	return misc_ask_question_ret;
 }
@@ -72,12 +85,13 @@ dialog_check_overwrite_file (const gchar *fname, GtkWidget *parent)
 	if (g_file_test (fname, G_FILE_TEST_ISFILE | G_FILE_TEST_ISLINK))
 	{
 		stat (fname, &st);
-		store_full_time_in_string (st.st_mtime, modified);
+		misc_string_store_full_time_in_string (st.st_mtime, modified);
 		question = g_strdup_printf 
 			(_("File %s will be overwritten.\n"
 			   "(%li bytes, modified %s)\n"
 			   "If you choose yes, the contents will be lost.\n\n"
-			   "Do you want to continue?"), fname, st.st_size, 
+			   "Do you want to continue?"), fname, 
+			 (unsigned long) st.st_size, 
 			 modified);
 		ret = dialog_ask_question (question, parent);
 		g_free (question);
@@ -162,12 +176,30 @@ dialog_confirm_check (GtkWidget *parent, const gchar *dialogTitle,
 void
 dialog_set_parent (GtkWidget *dialog, GtkWidget *parent_window)
 {
-	/* fullscreen mode flag */
-	extern gboolean fullscreen_active;
+	GaleonWindow *window = NULL;
 
-	/* Won't get far if this is true... */
-	if (dialog == NULL) return;
+	/* won't get far if this is true... */
+	if (dialog == NULL)
+	{
+		return;
+	}
 
+	/* find GaleonWindow */
+	if (parent_window != NULL)
+	{
+		window = gtk_object_get_data
+			(GTK_OBJECT (parent_window), "GaleonWindow");
+	}
+
+	/* check if we're in fullscreen mode and stacked above dock */
+	if (window != NULL && window->fullscreen_active && 
+	    eel_gconf_get_boolean (CONF_WINDOWS_FULLSCREEN_STACKING))
+	{
+		/* get to the top (must be shown first) */
+		gtk_widget_show (dialog);
+		gnome_win_hints_set_layer (dialog, WIN_LAYER_ABOVE_DOCK);
+	}
+	
 	/* if it's not a toplevel app (like e.g. the bm editor) */
 	if (!(GNOME_IS_APP (dialog)) && parent_window != NULL)
 	{
@@ -176,15 +208,6 @@ dialog_set_parent (GtkWidget *dialog, GtkWidget *parent_window)
 					      GTK_WINDOW (parent_window));
 	}
 
-	/* check if we're in fullscreen mode and stacked above dock */
-	if (fullscreen_active && 
-	    eel_gconf_get_boolean (CONF_WINDOWS_FULLSCREEN_STACKING))
-	{
-		/* get to the top (must be shown first) */
-		gtk_widget_show (dialog);
-		gnome_win_hints_set_layer (dialog, WIN_LAYER_ABOVE_DOCK);
-	}
-
 	/* finally show it and raise it to the top */
 	gtk_widget_show (dialog);
 	gdk_window_raise (dialog->window);
