@@ -57,7 +57,7 @@ downloader_save_link (GaleonEmbed *embed, const gchar *url)
 	case DOWNLOADER_BUILTIN:
 		embed_save_url (embed, (gchar *) url, CONF_DOWNLOADING_DIR,
 			  eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR),
-			  FALSE, FALSE, FALSE);
+			  FALSE, FALSE, FALSE, FALSE);
 		break;
 		
 	case DOWNLOADER_GTM:
@@ -73,7 +73,17 @@ downloader_save_link (GaleonEmbed *embed, const gchar *url)
 void
 downloader_external_save_url (const gchar *url)
 {
-	if (eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
+	Downloader downloader;
+	gchar *command;
+
+	/* determine if its necessary to ask for download dir even if 
+	 * the preference says so */
+	downloader = eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER);
+	command = eel_gconf_get_string (CONF_DOWNLOADING_EXTERNAL_COMMAND);
+	
+	if (eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR) &&
+	    (downloader != DOWNLOADER_EXTERNAL ||
+	     (command && strstr (command,"%f"))))
 	{
 		gchar *dirName, *retPath = NULL;
 		gboolean ret;
@@ -98,6 +108,7 @@ downloader_external_save_url (const gchar *url)
 	{
 		downloader_save_url_without_dir (url);
 	}
+	g_free (command);
 }
 
 static void
