@@ -287,6 +287,11 @@ embed_create_no_window (void)
 	embed->mozembed = gtk_moz_embed_new ();
 	g_assert (GTK_IS_MOZ_EMBED (embed->mozembed));
 
+	/* voodoo that prevents random crashes on embed close with moz 1.3b+ */
+#if (MOZILLA_SNAPSHOT > 4) && (MOZILLA_SNAPSHOT < 8)
+	GTK_WIDGET_SET_FLAGS(embed->mozembed, GTK_NO_WINDOW);
+#endif
+
 	/* ref/unref widget -- is this really needed? */
 	/* NB: apparently it is to get rid of some warnings, but I'm
 	 * still not clear why -- MattA 13/4/2001 */
@@ -536,6 +541,10 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 		}
 		if (shifted) {
 			downloader_save_link (previous, url);
+		} else if (alted || ctrled) {
+			new = embed_create_after_embed (previous,
+					TRUE, url,
+					EMBED_CREATE_GRAB_FOCUS);
 		} else {
 			embed_load_url (previous, url);
 		}
@@ -982,7 +991,7 @@ void
 embed_save_image (GaleonEmbed *embed, gchar *url, gboolean ask)
 {
 	embed_save_url (embed, url, CONF_STATE_SAVE_IMAGE_DIR, 
-			ask, FALSE, FALSE, TRUE);
+			ask, FALSE, FALSE, TRUE, TRUE);
 }
 
 /**
@@ -1026,7 +1035,7 @@ void embed_save_document (GaleonEmbed *embed, gboolean main)
 	}
 
 	/* save it */
-	embed_save_url (embed, url, CONF_STATE_SAVE_DIR, TRUE, TRUE, main, TRUE);
+	embed_save_url (embed, url, CONF_STATE_SAVE_DIR, TRUE, TRUE, main, TRUE, TRUE);
 
 	/* free */
 	g_free (url);
@@ -1040,7 +1049,7 @@ void
 embed_save_url (GaleonEmbed *embed, gchar *target, 
 		gchar *default_dir_pref, gboolean ask,
 		gboolean ask_content, gboolean main, 
-		gboolean save_dir)
+		gboolean save_dir, gboolean silent)
 {
 	GnomeVFSURI *uri;
 	gchar *retPath = NULL;
@@ -1124,7 +1133,8 @@ embed_save_url (GaleonEmbed *embed, gchar *target,
 			else
 			{
 				ret = mozilla_save_url (embed, target, 
-							retPath, ACTION_NONE,
+							retPath,
+							silent ? ACTION_SILENT : ACTION_NONE,
 							NULL);
 			}
 			embed_set_save_message (embed, retPath, ret);
@@ -1162,7 +1172,7 @@ embed_save_with_content (GaleonEmbed *embed, char *path, gboolean main)
 	if (mkdir (data_dir, 0755) != -1)
 	{
 		ret = mozilla_save_document (embed, path, data_dir,
-					     ACTION_NONE, main); 
+					     ACTION_SILENT, main); 
 	}
 	else 
 	{
@@ -1472,6 +1482,8 @@ embed_update_tab_title (GaleonEmbed *embed)
 		return;
 #endif
 
+	if(!GTK_IS_LABEL (embed->notebook_label)) return;
+
 	/* get the parent window */
 	return_if_not_sane_embed (embed);
 	window = embed->parent_window;
@@ -1531,6 +1543,8 @@ embed_update_tab_size (GaleonEmbed *embed)
 		return;
 #endif
 
+	if(!GTK_IS_LABEL (embed->notebook_label)) return;
+
 	/* get the parent window */
 	return_if_not_sane_embed (embed);
 	window = embed->parent_window;
@@ -1578,6 +1592,8 @@ embed_update_tab_status (GaleonEmbed *embed)
 		return;
 #endif
 
+	if(!GTK_IS_LABEL (embed->notebook_label)) return;
+
 	/* get the label */
 	label = GTK_WIDGET (embed->notebook_label);
 	g_return_if_fail (GTK_IS_LABEL (label));
