@@ -22,6 +22,7 @@
 #include "misc_string.h"
 #include "history.h"
 #include "eel-gconf-extensions.h"
+#include "window.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
@@ -107,7 +108,7 @@ embed_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 			else
 			{
 				embed_create_after_embed (embed, !tabbed,
-					real_uri, EMBED_CREATE_RAISE_WINDOW);
+					real_uri, 0);
 			}
 			
 			/* free allocated string */
@@ -136,5 +137,14 @@ void
 embed_tab_close_button_clicked_cb (GtkWidget *button, GaleonEmbed *embed)
 {
 	return_if_not_embed (embed);
-	embed_close (embed);
+
+	if((embed->parent_window) &&
+	   (g_list_length (embed->parent_window->embed_list) == 1))
+	{
+		window_close(embed->parent_window);
+	}
+	else
+	{
+		embed_close (embed);
+	}
 }
