@@ -68,6 +68,8 @@ static gboolean find_go (FindDialog *find_dialog,
 			 gboolean backwards);
 static void find_lookup_widgets (FindDialog *find, GladeXML *gxml);
 static void find_controls_set_sensitive (FindDialog *find);
+static gboolean find_next_or_prev (GaleonEmbed *embed, const gchar *string, 
+		                   gboolean backwards);
 
 /**
  * embed_show_find_dialog: show the find dialog
@@ -316,7 +318,7 @@ find_go (FindDialog *find_dialog,
 	}
 
 	/* do the find */
-	result = mozilla_find (embed, properties);
+	result = mozilla_find (embed, properties, TRUE);
 
 	/* free structure */
 	g_free (properties->search_string);
@@ -332,13 +334,32 @@ find_go (FindDialog *find_dialog,
  */
 gboolean
 find_next (GaleonEmbed *embed, const gchar *string)
+{
+	return find_next_or_prev (embed, string, FALSE);
+}
+
+/**
+ * find_previous: find the previous occurance of a string
+ */
+gboolean
+find_previous (GaleonEmbed *embed, const gchar *string)
+{
+	return find_next_or_prev (embed, string, TRUE);
+}
+
+/**
+ * find_next_or_prev: find the next (or previous) occurance of a string, used
+ * by smart bookmarks phrase searching
+ */
+static gboolean
+find_next_or_prev (GaleonEmbed *embed, const gchar *string, gboolean backwards)
 {
 	WrapperSearchProperties *properties;
 	gboolean result;
 
 	properties = g_new0 (WrapperSearchProperties, 1);
 	properties->search_string = g_strdup (string);
-	properties->backwards = FALSE;
+	properties->backwards = backwards;
 	properties->match_case =
 		eel_gconf_get_boolean (CONF_FIND_MATCH_CASE);
 	properties->entire_word =
@@ -348,7 +369,7 @@ find_next (GaleonEmbed *embed, const gchar *string)
 	properties->wrap =
 		eel_gconf_get_boolean (CONF_FIND_AUTOWRAP);
 
-	result = mozilla_find (embed, properties);
+	result = mozilla_find (embed, properties, FALSE);
 
 	g_free (properties->search_string);
 	g_free (properties);
