@@ -58,7 +58,8 @@ typedef enum
 {
 	DOCK_NONE,
 	DOCK_BOOKMARKS,
-	DOCK_HISTORY
+	DOCK_HISTORY,
+	DOCK_SIDEBAR
 } DockType;
 
 /**
