@@ -108,7 +108,7 @@ window_menu_find_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 }
 
 /**
- * window_menu_find_next_activate_cb: show find dialog
+ * window_menu_find_next_activate_cb: find next occurence
  */
 void
 window_menu_find_next_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
@@ -129,6 +129,28 @@ window_menu_find_next_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 		g_free (val);
 	}
 }
+/**
+ * window_menu_find_previous_activate_cb: find previous occurence
+ */
+void
+window_menu_find_previous_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GaleonEmbed *embed;
+	gchar *val, *utf8;
+
+	return_if_not_window (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	val = eel_gconf_get_string (CONF_FIND_WORD);
+	if (val)
+	{
+		utf8 = mozilla_utf8_to_locale (val);
+		find_previous (embed, utf8);
+		g_free (utf8);
+		g_free (val);
+	}
+}
 
 /**
  * window_menu_select_all_activate_cb: select all text on the current page
@@ -411,18 +433,21 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 				    GaleonWindow *window)
 {
 	static gchar *authors[] = {
-		"Marco Pesenti Gritti <mpeseng@tin.it>",
-		"Remco de Jong <rdj@rdj.cg.nu>",
+		"Philip Langdale <philipl@mail.utexas.edu>",
+		"Tommi Komulainen <tommi.komulainen@iki.fi>",
+		"Christophe Fergeau <teuf@users.sourceforge.net>",
 		/* if your encoding allows it, use aacute (U00E1)
 		 * for the 'a' of 'Fernandez' */
 		N_("Ricardo Fernandez Pascual <ric@users.sourceforge.net>"),
+		/* this one is supposed to be in cyrillic */
+		N_("Yanko Kaneti <yaneti@declera.com>"),
+		" ",
+		"Marco Pesenti Gritti <mpeseng@tin.it>",
+		"Remco de Jong <rdj@rdj.cg.nu>",
 		"Nate Case <nd@kracked.com>",
 		"Bob Phan <ex-freek@yahoo.com>",
 		"Matt Aubury <matt@ookypooky.com>",
 		"Joshua Horvath <jmhorvat@home.com>",
-		/* this one is supposed to be in cyrillic */
-		N_("Yanko Kaneti <yaneti@declera.com>"),
-		"Philip Langdale <philipl@mail.utexas.edu>",
 		"Matthew Mueller <donut@azstarnet.com>",
 		"Daniel Erat <dan_erat@pobox.com>",
 		"Jorn Baayen <jorn@nl.linux.org>",
@@ -500,6 +525,50 @@ window_menu_help_manual_activate_cb (GtkMenuItem *menuitem,
 	gnome_help_goto (NULL, "ghelp:galeon-manual");
 }
 
+/**
+ * window_menu_help_homepage_activate_cb: go to the Galeon homepage
+ */
+void
+window_menu_help_homepage_activate_cb (GtkWidget *menuitem, 
+				       GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, GALEON_HOMEPAGE_URL);
+}
+
+/**
+ * window_menu_help_faq_activate_cb: go to the Galeon FAQ page
+ */
+void
+window_menu_help_faq_activate_cb (GtkWidget *menuitem, 
+				  GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, GALEON_FAQ_URL);
+}
+
+/**
+ * window_menu_help_bugreport_activate_cb: go to the Galeon bug report page
+ */
+void
+window_menu_help_bugreport_activate_cb (GtkWidget *menuitem, 
+					GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, GALEON_BUGREPORT_URL);
+}
+
+/**
+ * window_menu_help_about_plugins_activate_cb: go to about:plugins
+ */
+void
+window_menu_help_about_plugins_activate_cb (GtkWidget *menuitem, 	
+					    GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_load_url (window, "about:plugins");
+}
+
 /**
  * window_menu_help_generic_release_cb: 
  */
@@ -536,8 +605,7 @@ window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
 
 	embed = embed_create_after_embed (window->active_embed, TRUE, NULL,
 					  EMBED_CREATE_LOAD_DEFAULT_URL |
-					  EMBED_CREATE_FORCE_JUMP |
-					  EMBED_CREATE_RAISE_WINDOW);
+					  EMBED_CREATE_FORCE_JUMP);
 
 	return_if_not_window (embed->parent_window);
 	window_grab_location_focus (embed->parent_window);
@@ -556,8 +624,7 @@ window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
 	window_grab_location_focus (window);
 	embed = embed_create_after_embed (window->active_embed, FALSE, NULL,
 					  EMBED_CREATE_LOAD_DEFAULT_URL |
-					  EMBED_CREATE_FORCE_JUMP |
-					  EMBED_CREATE_RAISE_WINDOW);
+					  EMBED_CREATE_FORCE_JUMP);
 }
 
 /** 
@@ -1285,14 +1352,21 @@ window_menu_open_session_activate_cb (GtkWidget *menuitem,
 }
 
 void
-window_menu_show_java_console_cb (GtkWidget *meniutem,
+window_menu_show_sidebar_cb (GtkWidget *menuitem,
+			     GaleonWindow *window)
+{
+	mozilla_show_sidebar (window);
+}
+
+void
+window_menu_show_java_console_cb (GtkWidget *menuitem,
 				  GaleonWindow *window)
 {
 	mozilla_show_java_console ();
 }
 
 void
-window_menu_show_javascript_console_cb (GtkWidget *meniutem,
+window_menu_show_javascript_console_cb (GtkWidget *menuitem,
 					GaleonWindow *window)
 {
 	GaleonEmbed *embed;
