@@ -538,7 +538,13 @@ GnomeUIInfo tools_menu_uiinfo[] =
 	  window_menu_history_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_H, GDK_CONTROL_MASK, NULL },
-	GNOMEUIINFO_SEPARATOR, 
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Mozilla Sidebar..."),
+	  N_("Open the Mozilla Sidebar"),
+	  window_menu_show_sidebar_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Java console..."),
 	  N_("Open the Java console"),
 	  window_menu_show_java_console_cb, NULL, NULL,
