@@ -31,8 +31,16 @@ void window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
                                              GaleonWindow *window);
 void window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 					 GaleonWindow *window);
+void window_menu_help_faq_activate_cb (GtkWidget *menuitem,
+				       GaleonWindow *window);
 void window_menu_help_manual_activate_cb (GtkMenuItem *menuitem,
 					  GaleonWindow *window);
+void window_menu_help_homepage_activate_cb (GtkWidget *menuitem,
+					    GaleonWindow *window);
+void window_menu_help_bugreport_activate_cb (GtkWidget *menuitem,
+					     GaleonWindow *window);
+void window_menu_help_about_plugins_activate_cb (GtkWidget *menuitem,
+					         GaleonWindow *window);
 void window_menu_help_generic_release_cb (GtkWidget *menuitem,
 					  GdkEventButton *event,
 					  GaleonWindow *window);
@@ -126,6 +134,8 @@ void window_menu_find_activate_cb (GtkMenuItem *menuitem,
                                    GaleonWindow *window);
 void window_menu_find_next_activate_cb (GtkMenuItem *menuitem,
                                         GaleonWindow *window);
+void window_menu_find_previous_activate_cb (GtkMenuItem *menuitem,
+                                            GaleonWindow *window);
 void window_menu_select_all_activate_cb (GtkMenuItem *menuitem,
                                          GaleonWindow *window);
 gboolean window_menu_edit_expose_event_cb (GtkWidget *editmenu, 
@@ -180,11 +190,13 @@ void window_menu_open_session_activate_cb  (GtkWidget *menuitem,
 					    GaleonWindow *window);
 void window_menu_bookmarks_toolbars_activate_cb (GtkCheckMenuItem *menuitem,
 						 GaleonWindow *window);
-void window_menu_show_java_console_cb (GtkWidget *meniutem,
+void window_menu_show_sidebar_cb (GtkWidget *menutem,
+				  GaleonWindow *window);
+void window_menu_show_java_console_cb (GtkWidget *menuitem,
 				       GaleonWindow *window);
 void window_menu_show_javascript_console_cb (GtkWidget *meniutem,
 					     GaleonWindow *window);
-void window_menu_show_autoreload_cb (GtkWidget *meniutem,
+void window_menu_show_autoreload_cb (GtkWidget *menuitem,
 				     GaleonWindow *window);
 void window_menu_show_page_info_cb (GtkWidget *menuitem,
 				    GaleonWindow *window);
