@@ -22,11 +22,15 @@
 
 #include "galeon.h"
 #include "mime.h"
-#include "misc.h"
+#include "misc_general.h"
 
 #include <string.h>
+#include <libgnome/libgnome.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
+#include <gnome-xml/parser.h>
+#include <gnome-xml/tree.h>
+#include <gnome-xml/xmlmemory.h>
 
 /* the global list of MimeItems */
 GList *mime_types = NULL;
@@ -254,7 +258,7 @@ void mime_db_load (void)
 	/* load the file */
 	doc = xmlParseFile (filename);
 	g_free (filename);
-	if (doc == NULL) 
+	if (doc == NULL || doc->root == NULL) 
 	{
 		g_warning ("unable to parse `%s', resetting "
 			   "to defaults ", filename);
