@@ -979,7 +979,7 @@ mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 {
 	GaleonWindow *window;
 	int state = 0;
-	gboolean handled = FALSE;
+	gboolean handled = FALSE, bypassed = FALSE;
 	WrapperKeyEventInfo *info;
 
 	return_val_if_not_sane_embed (embed, FALSE);
@@ -1094,31 +1094,9 @@ mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	/* do not pass keypresses without modifiers in textareas to the
 	 * gtk accel table */
 	if ((info->modifier == 0) && (info->ctx.context & CONTEXT_INPUT))
-		handled = TRUE;
-
-	if ((handled == FALSE) && (info->modifier == 0))
 	{
-		switch (info->gdk_keyval)
-		{
-		case GDK_k:
-			mozilla_scroll_up (embed);
-			handled = TRUE;
-			break;
-		case GDK_j:
-			mozilla_scroll_down (embed);
-			handled = TRUE;
-			break;
-		case GDK_h:
-			mozilla_scroll_left (embed);
-			handled = TRUE;
-			break;
-		case GDK_l:
-			mozilla_scroll_right (embed);
-			handled = TRUE;
-			break;
-		default:
-			break;
-		}
+		handled = TRUE;
+		bypassed = TRUE;
 	}
 
 	/* We haven't specifically handled the keypress, so send it to the
@@ -1134,7 +1112,7 @@ mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	g_free (info);
 				
 	/* If it wasn't an accelerator, let Mozilla handle it */
-	return handled;
+	return bypassed ? FALSE : handled;
 }
 
 /**
