@@ -16,11 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "prefs.h"
 #include "outputbuffer.h"
 
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <libgnome/gnome-i18n.h>
 
 #include "nsIIOService.h"
@@ -57,7 +65,8 @@ NS_IMETHODIMP GBaseHelpProtocolHandler::NewURI(const nsACString &aSpec,
 						getter_AddRefs(gnomeHelpURI));
 	if (NS_FAILED(rv)) return rv;
 
-	if (aSpec == NS_LITERAL_CSTRING("info:dir"))
+	nsCAutoString cSpec(aSpec);
+	if (cSpec.Equals("info:dir"))
 		rv = gnomeHelpURI->SetSpec(NS_LITERAL_CSTRING("toc:info"));
 	else
 		rv = gnomeHelpURI->SetSpec(aSpec);
@@ -82,6 +91,11 @@ NS_IMETHODIMP GBaseHelpProtocolHandler::NewChannel(nsIURI *aURI,
 	rv = aURL->GetFilePath (mDocName);
 	if (NS_FAILED(rv)) return rv;
 
+	//Converters are badly behaved and seem to invoke the shell with the
+	//document name as given. Shell control chars, if processed are a bad
+	//exploit. So we strip them out.
+	mDocName.StripChars("!;:&|^");
+
 	rv = CreatePage ();
 	NS_IF_ADDREF (*_retval = mChannel);
 	return rv;
@@ -91,7 +105,7 @@ NS_METHOD GBaseHelpProtocolHandler::CreateGHelpURI (nsIURI **_retval)
 {
 	nsresult rv;
 
-	nsCString cSpec = mScheme + NS_LITERAL_CSTRING(":") + mDocName;
+	nsCAutoString cSpec(mScheme + NS_LITERAL_CSTRING(":") + mDocName);
 	if ('.' == cSpec.CharAt(cSpec.Length()-2))
 	{
 		char cat = cSpec.CharAt(cSpec.Length()-1);
@@ -118,16 +132,16 @@ NS_METHOD GBaseHelpProtocolHandler::CreatePage (void)
 	gchar *argv[6];
 	argv[0] = g_strdup (mConverter.get());
 	argv[1] = g_strdup (mDocName.get());
-	if (mScheme == NS_LITERAL_CSTRING("info"))
+	if (mScheme.Equals("info"))
 	{
-		if (mSection.Length() == 0)
+		if (mSection.IsEmpty())
 			mSection = NS_LITERAL_CSTRING("Top");
 		argv[2] = "-a";
 		argv[3] = g_strdup (mSection.get());
 		argv[4] = "-g";	//Activate galeon mode in gnome-info2html2
 		argv[5] = NULL;
 	}
-	else if (mScheme == NS_LITERAL_CSTRING("man"))
+	else if (mScheme.Equals("man"))
 	{
 		argv[2] = "-n";
 		argv[3] = g_strconcat ("man:", mDocName.get(), NULL);
@@ -143,8 +157,7 @@ NS_METHOD GBaseHelpProtocolHandler::CreatePage (void)
 	int ret = getOutputFrom(argv, NULL, 0, &outbuf, &outbuflen);
 	g_free (argv[0]);
 	g_free (argv[1]);
-	if (mScheme == NS_LITERAL_CSTRING("info") ||
-	    mScheme == NS_LITERAL_CSTRING("man"))
+	if (mScheme.Equals("info") || mScheme.Equals("man"))
 		g_free (argv[3]);
 
 	if (ret == -1)
@@ -187,18 +200,25 @@ NS_METHOD GBaseHelpProtocolHandler::CreatePage (void)
 	g_free (outbuf);
 	if (NS_FAILED(rv)) return rv;
 
-	/* finish the rendering */
-	nsCOMPtr<nsIInputStream> iStream;
-	PRUint32 size;  
-    
+        /* finish the rendering */
+        nsCOMPtr<nsIInputStream> iStream;
+
+        rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
+        if (NS_FAILED(rv)) return rv;
+
+#if MOZILLA_SNAPSHOT > 4
+	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
+				      iStream, NS_LITERAL_CSTRING("text/html"),
+				      NS_LITERAL_CSTRING("utf-8"));
+#else
+	PRUint32 size;
 	rv = sStream->GetLength(&size);
 	if (NS_FAILED(rv)) return rv;
 
-	rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
-	if (NS_FAILED(rv)) return rv;
-
 	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
-				      iStream, "text/html", size);
+				      iStream, NS_LITERAL_CSTRING("text/html"),
+				      NS_LITERAL_CSTRING("utf-8"), size);
+#endif
 	if (NS_FAILED(rv)) return rv;
 
 	return rv;
