@@ -16,6 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H   
+#include <config.h>
+#endif
+
 #include "nsCOMPtr.h"
 #include "nsIURI.h"
 #include "nsIChannel.h"
@@ -62,12 +66,17 @@ NS_IMETHODIMP GBaseProtocolContentHandler::NewChannel(nsIURI *aURI,
 	rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
 	if (NS_FAILED(rv)) return rv;
 
-	nsCOMPtr<nsIChannel> aChannel;
-	rv = NS_NewInputStreamChannel(getter_AddRefs(aChannel), aURI,
+	nsCOMPtr<nsIChannel> channel;
+#if MOZILLA_SNAPSHOT > 4
+	rv = NS_NewInputStreamChannel(getter_AddRefs(channel), aURI,
+				      iStream, mMimeType, NS_LITERAL_CSTRING(""));
+#else
+	rv = NS_NewInputStreamChannel(getter_AddRefs(channel), aURI,
 				      iStream, mMimeType, NS_LITERAL_CSTRING(""), 0);
+#endif
 	if (NS_FAILED(rv)) return rv;
 
-	NS_IF_ADDREF (*_retval = aChannel);
+	NS_IF_ADDREF (*_retval = channel);
 	return rv;
 }
 
