@@ -143,10 +143,6 @@
  * really messy.
  */
 
-extern "C" {
-#include "libgnomevfs/gnome-vfs-mime-handlers.h"
-}
-
 #include "galeon.h"
 #include "mime.h"
 #include "window.h"
@@ -173,11 +169,9 @@ extern "C" {
 
 #include "nsCRT.h"
 #include "nsCOMPtr.h"
-#include "nsIFactory.h"
 #include "nsISupportsArray.h"
 #include "nsIServiceManager.h"
 #include "nsWeakReference.h"
-#include "nsXPComFactory.h"
 
 #include "nsString.h"
 #include "nsIURI.h"
@@ -226,13 +220,15 @@ struct MimeAskActionDialog
 
 struct HelperAppChooserDialog
 {
-	HelperAppChooserDialog (void) : mOtherHelperApp(FALSE) {};
+	HelperAppChooserDialog (void) : mOtherHelperApp(FALSE),
+					mDefaultHelper(NULL) {};
 	~HelperAppChooserDialog () {};
 
+	gboolean mOtherHelperApp;
 	GList *mHelperApps;
+	GnomeVFSMimeApplication *mDefaultHelper;
 	GContentHandler *mContentHandler;
 	GladeXML *mGXML;
-	gboolean mOtherHelperApp;
 
 	GtkWidget *mDialogWidget;
 	GtkWidget *mHelperGnomeEntry;
@@ -246,15 +242,14 @@ struct HelperAppChooserDialog
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
-GContentHandler::GContentHandler()
+GContentHandler::GContentHandler() : mUri(nsnull),
+				     mMimeType(nsnull),
+				     mMimeItem(nsnull),
+				     mDownloadCanceled(PR_FALSE),
+				     mHelperProgress(PR_FALSE)
 {
 	NS_INIT_ISUPPORTS();
 	/* member initializers and constructor code */
-	mUri = nsnull;
-	mMimeType = nsnull;
-	mMimeItem = nsnull;
-	mDownloadCanceled = PR_FALSE;
-	mHelperProgress = PR_FALSE;
 }
 
 GContentHandler::~GContentHandler()
@@ -270,14 +265,19 @@ GContentHandler::~GContentHandler()
 
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
-					      nsISupports *aContext)
+					      nsISupports *aContext
+#if MOZILLA_SNAPSHOT > 9
+, PRBool aForced)
+#else
+)
+#endif
 {
 	nsresult rv;
 
 	mLauncher = aLauncher;
 	mContext = aContext;
 	rv = Init ();
-	
+
 	switch (mMimeItem->action)
 	{
 	 case MIME_UNKNOWN:
@@ -287,7 +287,6 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 		break;
 	 case MIME_IGNORE:
 		mLauncher->Cancel ();
-		delete mMimeItem;
 		break;
 	 case MIME_SAVE_TO_DISK:
 	 	if (eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER)
@@ -312,12 +311,17 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 	return NS_OK;
 }
 
+//This function is supposed to be depreciated, with the external helper app service
+//invoking nsIFilePicker itself. But it's still calling it. Go figure.
 /* nsILocalFile promptForSaveToFile (in nsISupports aWindowContext, in wstring aDefaultFile, in wstring aSuggestedFileExtension); */
-NS_IMETHODIMP GContentHandler::
-		PromptForSaveToFile(nsISupports *aWindowContext,
-				    const PRUnichar *aDefaultFile,
-				    const PRUnichar *aSuggestedFileExtension,
-				    nsILocalFile **_retval)
+NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
+#if MOZILLA_SNAPSHOT > 10 
+                                                   nsIHelperAppLauncher *aLauncher,
+#endif
+						   nsISupports *aWindowContext,
+						   const PRUnichar *aDefaultFile,
+						   const PRUnichar *aSuggestedFileExtension,
+						   nsILocalFile **_retval)
 {
 	nsresult rv;
 
@@ -360,16 +364,16 @@ NS_IMETHODIMP GContentHandler::
 	}
 
 	saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	saveDir->InitWithPath (expanded_directory);
+	saveDir->InitWithNativePath (nsDependentCString(expanded_directory));
 	g_free (expanded_directory);
 
 	nsCOMPtr<nsIFilePicker> aFilePicker =
 				do_CreateInstance (G_FILEPICKER_CONTRACTID);
 
-	nsString title = NS_LITERAL_STRING (_("Select the destination "
-					      "filename"));
+//	NS_NAMED_LITERAL_STRING (title, _("Select the destination filename"));
+	NS_NAMED_LITERAL_CSTRING(title, _("Select the destination filename"));
 
-	aFilePicker->Init (aWindowInternal, title.get(), 
+	aFilePicker->Init (aWindowInternal, PromiseFlatString(NS_ConvertUTF8toUCS2(title)).get(),
 			   nsIFilePicker::modeSave);
 	aFilePicker->SetDefaultString (aDefaultFile);
 	aFilePicker->SetDisplayDirectory (saveDir);
@@ -388,18 +392,16 @@ NS_IMETHODIMP GContentHandler::
 	}
 	else
 	{
-		char *cDirName;
-		saveDir->GetPath (&cDirName);
+		nsCAutoString cDirName;
+		saveDir->GetNativePath (cDirName);
 		char *cDefaultFile = mozilla_unicode_to_locale (aDefaultFile);
-		char *cFileName = g_strconcat (cDirName, "/", cDefaultFile, NULL);
-		aFile->InitWithPath (cFileName);
+		aFile -> InitWithNativePath (cDirName +
+					     NS_LITERAL_CSTRING("/") +
+					     nsDependentCString(cDefaultFile));
 		g_free (cDefaultFile);
-		g_free (cFileName);
-		nsMemory::Free (cDirName);
 		okToSave = nsIFilePicker::returnOK;
 	};
 
-
 	if (okToSave == nsIFilePicker::returnCancel)
 		return NS_ERROR_FAILURE;
 	else
@@ -408,8 +410,8 @@ NS_IMETHODIMP GContentHandler::
 		rv = aFile->GetParent (getter_AddRefs(aDirectory));
 		if(NS_SUCCEEDED(rv) && aDirectory)
 		{
-			nsXPIDLCString cDirectoryName;
-			rv = aDirectory->GetPath(getter_Copies(cDirectoryName));
+			nsCAutoString cDirectoryName;
+			rv = aDirectory->GetNativePath(cDirectoryName);
 			if(NS_SUCCEEDED(rv))
 			{
 				eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR,
@@ -422,43 +424,59 @@ NS_IMETHODIMP GContentHandler::
 	};
 }
 
+#if MOZILLA_SNAPSHOT <= 9
 /* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::
 			ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 					   nsISupports *aContext)
 {
+#if MOZILLA_SNAPSHOT > 1
+	g_print ("GContentHandler::ShowProgressDialog is depreciated!\n");
+	return NS_ERROR_NOT_IMPLEMENTED;
+#else
 	DownloadAction aAction =
 		mHelperProgress ? ACTION_SAVEFORHELPER : ACTION_NONE;
 	GProgressListener2 *aListener = new GProgressListener2 ();
 	aListener->InitForDownload (aLauncher, aContext, this, aAction);
 	return NS_OK;
+#endif
 }
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // begin local public methods impl
 ////////////////////////////////////////////////////////////////////////////////
 
-NS_IMETHODIMP GContentHandler::LaunchExternalDownloader (void)
+NS_METHOD GContentHandler::LaunchExternalDownloader (void)
 {
 	downloader_external_save_url (mUrl.get());
 	return NS_OK;
 }
 
 
-NS_IMETHODIMP GContentHandler::FindHelperApp (void)
+NS_METHOD GContentHandler::FindHelperApp (void)
 {
 	if (mMimeItem->alwaysUse)
 	{
 		if (mMimeItem->urlHelper)
 			return LaunchHelperApp ();
 		else
+#if MOZILLA_SNAPSHOT > 1
+		{
+			if (NS_SUCCEEDED(SynchroniseMIMEInfo()))
+				return mLauncher->LaunchWithApplication(nsnull, PR_FALSE);
+			else
+				return NS_ERROR_FAILURE;
+		}
+#else
 			return ShowHelperProgressDialog ();
+#endif
 	}
 	else
 		return ChooseHelperApp ();
 }
 
-NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
+NS_METHOD GContentHandler::LaunchHelperApp (void)
 {
 	gchar *tmp_dir = g_get_tmp_dir ();
 
@@ -475,51 +493,28 @@ NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 			appLauncher->DeleteTemporaryFileOnExit(mTempFile);
 		}
 
-		nsXPIDLCString aFileName;
-		nsCString document;
+		nsCString aFileName;
+		mTempFile->GetNativePath(aFileName);
 
-		mTempFile->GetPath(getter_Copies(aFileName));
-
-		PRBool freeMimeItem(PR_FALSE);
-		if (nsCRT::strcasecmp(mMimeItem->helperApp, "GNOME Default") == 0)
-		{
-			mMimeItem = mime_item_clone (mMimeItem);
-			SetHelperApp (gnome_vfs_mime_get_default_application (mMimeType),
-				      mMimeItem->alwaysUse);
-			freeMimeItem = PR_TRUE;
-		}
-
-		if (mMimeItem->urlHelper)
-		{
-			document = mUrl;
-		}
-		else
-		{
-			document = aFileName;
-		}
-		char *full_command = g_strconcat (mMimeItem->helperApp,
-						  " ", document.get(), NULL);
+		const nsCString &document = mMimeItem->urlHelper ? mUrl : aFileName;
 
 		if (mMimeItem->terminalHelper == FALSE)
 		{
-			char **argv = g_strsplit (full_command, " ", 100);
-			int i;
-			for (i = 0; argv[i] != NULL; i++);
-			gnome_execute_async (tmp_dir, i, argv);
-			g_strfreev (argv);
+			const char *argv[2] = { mMimeItem->helperApp,
+						document.get() };
+			gnome_execute_async (tmp_dir, 2,
+					     const_cast<char **>(argv));
 		}
 		else
 		{
-			char *argv[3] = { "gnome-terminal", "-e",
-					  full_command };
-			gnome_execute_async (tmp_dir, 3, argv);
+			const char *argv[4] = { "gnome-terminal", "-e",
+						mMimeItem->helperApp,
+						document.get() };
+			gnome_execute_async (tmp_dir, 4,
+					     const_cast<char **>(argv));
 		}
-		g_free (full_command);
 
-		if (freeMimeItem)
-		{
-			mime_item_free (mMimeItem);
-		}
+		if(mMimeItem->urlHelper) mLauncher->Cancel();
 	}
 	else
 	{
@@ -529,30 +524,39 @@ NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::ShowHelperProgressDialog (void)
+#if MOZILLA_SNAPSHOT <= 1
+NS_METHOD GContentHandler::ShowHelperProgressDialog (void)
 {
 	mHelperProgress = PR_TRUE;
 	return ShowProgressDialog (mLauncher,mContext);
 }
+#endif
 
-NS_IMETHODIMP GContentHandler::GetLauncher (nsIHelperAppLauncher * *_retval)
+NS_METHOD GContentHandler::GetLauncher (nsIHelperAppLauncher * *_retval)
 {
 	NS_IF_ADDREF (*_retval = mLauncher);
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::GetContext (nsISupports * *_retval)
+NS_METHOD GContentHandler::GetContext (nsISupports * *_retval)
 {
 	NS_IF_ADDREF (*_retval = mContext);
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
+NS_METHOD GContentHandler::GetMimeItemReference (MimeItem * *_retval)
+{
+	*_retval = mMimeItem;
+	return NS_OK;
+}
+
+NS_METHOD GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 					    PRBool alwaysUse)
 {
 	mMimeItem->helperApp = g_strdup (aHelperApp->command);
 	mMimeItem->alwaysUse = alwaysUse ? TRUE : FALSE;
 	mMimeItem->terminalHelper = aHelperApp->requires_terminal;
+
 	if (aHelperApp->expects_uris ==
 				GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS)
 	{
@@ -563,8 +567,7 @@ NS_IMETHODIMP GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 		mMimeItem->urlHelper = TRUE;
 	}
 
-	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") == 0 ||
-	    nsCRT::strcmp(aHelperApp->id,"GNOME Default") == 0)
+	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") == 0)
 	{
 		g_free (aHelperApp->command);
 		g_free (aHelperApp->id);
@@ -574,10 +577,56 @@ NS_IMETHODIMP GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 	return NS_OK;
 }
 
+NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
+{
+	nsresult rv;
+	nsCOMPtr<nsIMIMEInfo> mimeInfo;
+	rv = mLauncher->GetMIMEInfo(getter_AddRefs(mimeInfo));
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsILocalFile> helperFile;
+	rv = NS_NewNativeLocalFile(nsDependentCString(mMimeItem->helperApp),
+				   PR_TRUE,
+				   getter_AddRefs(helperFile));
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	rv = mimeInfo->SetPreferredApplicationHandler(helperFile);
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;	
+
+	nsMIMEInfoHandleAction mimeInfoAction;
+	switch(mMimeItem->action)
+	{
+	  case MIME_SAVE_TO_DISK:
+		mimeInfoAction = nsIMIMEInfo::saveToDisk;
+		break;
+	  case MIME_ASK_ACTION:
+		mimeInfoAction = nsIMIMEInfo::alwaysAsk;
+		break;
+	  case MIME_RUN_PROGRAM:
+		mimeInfoAction = nsIMIMEInfo::useHelperApp;
+		break;
+	  default:
+	  	g_warning ("Processing a download with invalid mime action!\n");
+	  	mimeInfoAction = nsIMIMEInfo::saveToDisk;
+	}
+
+	if(mMimeItem->terminalHelper) //Information passing kludge!
+	{
+		rv = mimeInfo->SetApplicationDescription
+				(NS_LITERAL_STRING("runInTerminal").get());
+		if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+	}
+
+	rv = mimeInfo->SetPreferredAction(mimeInfoAction);
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // begin local private methods impl
 ////////////////////////////////////////////////////////////////////////////////
-NS_IMETHODIMP GContentHandler::Init (void)
+NS_METHOD GContentHandler::Init (void)
 {
 	nsresult rv;
 
@@ -585,23 +634,22 @@ NS_IMETHODIMP GContentHandler::Init (void)
 	rv = mLauncher->GetMIMEInfo (getter_AddRefs(aMIMEInfo));
 	rv = aMIMEInfo->GetMIMEType (&mMimeType);
 	
+#if MOZILLA_SNAPSHOT > 11
+	rv = mLauncher->GetSource(getter_AddRefs(mUri));
+	rv = mLauncher->GetTargetFile(getter_AddRefs(mTempFile));
+#else
 	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
 					&mTimeDownloadStarted,
 					getter_AddRefs(mTempFile));
+#endif
+
 	rv = mUri->GetSpec (mUrl);
-#if 0
-	/* GetSource seems redundant and isn't in 0.9 This code is here while
-	   it remains unclear what GetSource is for. --phil */
-	nsCOMPtr<nsIURI> aUri;
-	rv = mLauncher->GetSource(getter_AddRefs(aUri));
-	rv = aUri->GetSpec (mUrl);
-#endif	
 	ProcessMimeInfo ();
 
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::ProcessMimeInfo (void)
+NS_METHOD GContentHandler::ProcessMimeInfo (void)
 {
 	if (mMimeType == NULL ||
 	    !nsCRT::strcmp(mMimeType, "application/octet-stream"))
@@ -610,7 +658,7 @@ NS_IMETHODIMP GContentHandler::ProcessMimeInfo (void)
 		nsCOMPtr<nsIURL> aURL = do_QueryInterface(mUri, &rv);
 		if (NS_SUCCEEDED(rv) && aURL)
 		{
-			nsCString uriFileName;
+			nsCAutoString uriFileName;
 			aURL->GetFileName(uriFileName);
 			mMimeType = g_strdup
 					(gnome_vfs_mime_type_from_name
@@ -632,7 +680,7 @@ NS_IMETHODIMP GContentHandler::ProcessMimeInfo (void)
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::MIMEAskAction (void)
+NS_METHOD GContentHandler::MIMEAskAction (void)
 {
 	MimeAskActionDialog *dialog = new MimeAskActionDialog;
 	GtkWidget *label;
@@ -658,11 +706,12 @@ NS_IMETHODIMP GContentHandler::MIMEAskAction (void)
 	return NS_OK;
 }
 
-NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
+NS_METHOD GContentHandler::ChooseHelperApp (void)
 {
 	HelperAppChooserDialog *dialog = new HelperAppChooserDialog;
 
 	dialog->mHelperApps = gnome_vfs_mime_get_all_applications (mMimeType);
+	dialog->mDefaultHelper = gnome_vfs_mime_get_default_application (mMimeType);
 	dialog->mContentHandler = this;
 	dialog->mGXML = glade_widget_new ("galeon.glade",
 					  "helper_chooser_dialog", 
@@ -689,7 +738,7 @@ NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
 
 	GtkWidget *menu = gtk_menu_new ();
 
-	if(gnome_vfs_mime_get_default_application (mMimeType))
+	if(dialog->mDefaultHelper)
 	{
 		GtkWidget *defaultItem = gtk_menu_item_new_with_label (
 						_("GNOME Default"));
@@ -734,30 +783,6 @@ NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
 	return NS_OK;
 }
 
-//------------------------------------------------------------------------------
-
-NS_DEF_FACTORY (GContentHandler, GContentHandler);
-
-/**
- * NS_NewContentHandlerFactory:
- */ 
-nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGContentHandlerFactory *result = new nsGContentHandlerFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // begin MIMEAskActionDialog callbacks.
 ////////////////////////////////////////////////////////////////////////////////
@@ -862,18 +887,22 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 			helper_chooser_cancel_clicked_cb (button, dialog);
 			return;
 		}
+
 		gpointer item = gtk_menu_get_active (GTK_MENU(menu));
 		gint index = g_list_index (menuItems,item);
-		if (index!=0)
+		if ((index == 0) && dialog->mDefaultHelper)
 		{
-			GList *helperApps = g_list_nth(dialog->mHelperApps,index-1);
+			helperApp = dialog->mDefaultHelper;
+		}
+		else if (!dialog->mDefaultHelper)
+		{
+			GList *helperApps = g_list_nth(dialog->mHelperApps,index);
 			helperApp = (GnomeVFSMimeApplication *)(helperApps->data);
 		}
 		else
 		{
-			helperApp = new GnomeVFSMimeApplication;
-			helperApp->id = g_strdup("GNOME Default");
-			helperApp->command = g_strdup(_("GNOME Default"));
+			GList *helperApps = g_list_nth(dialog->mHelperApps,index-1);
+			helperApp = (GnomeVFSMimeApplication *)(helperApps->data);
 		}
 	}
 	else
@@ -899,7 +928,25 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 				GTK_TOGGLE_BUTTON(dialog->mCheckAlwaysUse)) ?
 			   PR_TRUE : PR_FALSE;
 	dialog->mContentHandler->SetHelperApp (helperApp, alwaysUse);
-	dialog->mContentHandler->ShowHelperProgressDialog ();
+
+	MimeItem *aMimeItem(nsnull);
+	dialog->mContentHandler->GetMimeItemReference(&aMimeItem);
+
+	if(aMimeItem->urlHelper)
+		dialog->mContentHandler->LaunchHelperApp();
+	else
+	{
+#if MOZILLA_SNAPSHOT > 1
+		if (NS_SUCCEEDED(dialog->mContentHandler->SynchroniseMIMEInfo()))
+		{
+			nsCOMPtr<nsIHelperAppLauncher> aLauncher;
+			dialog->mContentHandler->GetLauncher(getter_AddRefs(aLauncher));
+			aLauncher->LaunchWithApplication(nsnull, PR_FALSE);
+		}
+#else
+		dialog->mContentHandler->ShowHelperProgressDialog ();
+#endif
+	}
 
 	gnome_vfs_mime_application_list_free (dialog->mHelperApps);
 
@@ -946,11 +993,12 @@ void helper_chooser_browse_clicked_cb (GtkButton *button,
 	nsCOMPtr<nsIFilePicker> aFilePicker =
 			do_CreateInstance (G_FILEPICKER_CONTRACTID);
 
-	nsString title = NS_LITERAL_STRING (N_("Choose Helper"
-					       " Application"));
+	PRUnichar *title = mozilla_locale_to_unicode (N_("Choose Helper"
+                                               " Application"));
 
-	aFilePicker->Init (aWindowInternal, title.get(), 
+	aFilePicker->Init (aWindowInternal, title, 
 			   nsIFilePicker::modeOpen);
+	g_free(title);
 	aFilePicker->SetDefaultString (nsnull);
 	aFilePicker->SetDisplayDirectory (aSaveDir);
 		
