@@ -16,6 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "EventContext.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDocument.h"
@@ -35,6 +39,7 @@
 #include "nsIDOMHTMLLabelElement.h"
 #include "nsIDOMHTMLLegendElement.h"
 #include "nsIDOMHTMLTextAreaElement.h"
+#include "nsCRT.h"
 
 #include "misc_string.h"
 
@@ -55,7 +60,7 @@ nsresult EventContext::Init (nsIDOMEvent *event, GaleonWrapper *wrapper)
 	return NS_OK;
 }
 
-#define MOZILLA_KEYS_LENGHT 16
+#define MOZILLA_KEYS_LENGHT 17
 
 static const struct 
 {
@@ -78,6 +83,7 @@ mozilla_keys [] =
 	{ CTRL_KEY, NS_VK_V + 32 },
 	{ CTRL_KEY + KEY_CODE, NS_VK_LEFT },
 	{ CTRL_KEY + KEY_CODE, NS_VK_RIGHT },
+	{ CTRL_KEY + KEY_CODE, NS_VK_HOME },
 	{ ALT_KEY + KEY_CODE, NS_VK_LEFT },
 	{ ALT_KEY + KEY_CODE, NS_VK_RIGHT }
 };
@@ -249,11 +255,15 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			rv = image->GetLongDesc (img);
 			if (NS_SUCCEEDED(rv) && !img.IsEmpty())
 			{
-				nsCString src, imglongdesc;
+				nsCAutoString src, imglongdesc;
 				src.AssignWithConversion(img);
 
 				nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+				uri = doc->GetDocumentURL();
+#else
 				doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
 				rv = uri->Resolve (src, imglongdesc);
 
@@ -295,9 +305,13 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 				nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+				uri = doc->GetDocumentURL();
+#else
 				doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
-				nsCString src, cImg;
+				nsCAutoString src, cImg;
 				src.AssignWithConversion(img);
 
 				rv = uri->Resolve (src, cImg);
@@ -338,9 +352,13 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 				nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+				uri = doc->GetDocumentURL();
+#else
 				doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 				
-				nsCString src, cImg;
+				nsCAutoString src, cImg;
 				src.AssignWithConversion(img);
 
 				rv = uri->Resolve (src, cImg);
@@ -486,7 +504,8 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					info->link = ToNewCString(href);
 				}
 			}
-			else if (tag.EqualsWithConversion("textarea", PR_TRUE))
+			else if (tag.EqualsWithConversion("textarea", PR_TRUE) ||
+				 tag.EqualsWithConversion("input", PR_TRUE))
 			{
 				info->context |= CONTEXT_INPUT;
 			}
@@ -504,11 +523,17 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				
 				if (!value.IsEmpty())
 				{
-					nsCString tmp, bgimg;
+					nsCAutoString tmp, bgimg;
 					tmp.AssignWithConversion(value);
 
-					nsIURI *uri;
-					doc->GetBaseURL(uri);
+					nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+					uri = doc->GetBaseURL();
+#elif MOZILLA_SNAPSHOT > 9
+					doc->GetBaseURL(getter_AddRefs(uri));
+#else
+					doc->GetBaseURL(*getter_AddRefs(uri));
+#endif
 					rv = uri->Resolve 
 						(tmp, bgimg);
 					info->bgimg = nsCRT::strdup(bgimg.get());
@@ -531,11 +556,17 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 					if (!value.IsEmpty())
 					{
-						nsCString tmp, bgimg;
+						nsCAutoString tmp, bgimg;
 						tmp.AssignWithConversion(value);
 
-						nsIURI *uri;
-						doc->GetBaseURL(uri);
+						nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+						uri = doc->GetBaseURL();
+#elif MOZILLA_SNAPSHOT > 9
+						doc->GetBaseURL(getter_AddRefs(uri));
+#else
+						doc->GetBaseURL(*getter_AddRefs(uri));
+#endif
 						rv = uri->Resolve 
 							(tmp, bgimg);
 						info->bgimg = nsCRT::strdup(bgimg.get());
@@ -555,11 +586,17 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				rv = GetCSSBackground (node, cssurl);
 				if (NS_SUCCEEDED (rv))
 				{
-					nsCString tmp, bgimg;
+					nsCAutoString tmp, bgimg;
 					tmp.AssignWithConversion(cssurl);
 
-					nsIURI *uri;
-					doc->GetBaseURL(uri);
+					nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+					uri = doc->GetBaseURL();
+#elif MOZILLA_SNAPSHOT > 9
+					doc->GetBaseURL(getter_AddRefs(uri));
+#else
+					doc->GetBaseURL(*getter_AddRefs(uri));
+#endif
 					rv = uri->Resolve 
 						(tmp, bgimg);
 					info->bgimg = nsCRT::strdup(bgimg.get());
@@ -577,7 +614,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	return NS_OK;
 }
 
-nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAutoString& url)
+nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsString& url)
 {
 	nsresult result;
 
@@ -627,13 +664,16 @@ nsresult EventContext::GetMouseEventInfo (WrapperMouseEventInfo *info)
 {
 	nsresult result;
 	DOMTimeStamp ts;
-	nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent*)mEvent;
+	nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent *) mEvent;
 
-	aMouseEvent->GetButton ((PRUint16*)&info->button);
+	aMouseEvent->GetButton ((PRUint16 *) &info->button);
 
 	aMouseEvent->GetTimeStamp(&ts);
 	info->timestamp = ts;
 
+	aMouseEvent->GetScreenX ((PRInt32 *) &info->x_root);
+	aMouseEvent->GetScreenY ((PRInt32 *) &info->y_root);
+
 	/* be sure we are not clicking on the scroolbars */
 
 	nsCOMPtr<nsIDOMNSEvent> nsEvent = do_QueryInterface(aMouseEvent, &result);
@@ -646,7 +686,7 @@ nsresult EventContext::GetMouseEventInfo (WrapperMouseEventInfo *info)
 	nsCOMPtr<nsIDOMNode> OriginalNode = do_QueryInterface(OriginalTarget);
 	if (!OriginalNode) return NS_ERROR_FAILURE;
 
-	nsString nodename;
+	nsAutoString nodename;
 	OriginalNode->GetNodeName(nodename);
 
 	if (nodename.EqualsWithConversion ("xul:scrollbarbutton") ||
