@@ -16,15 +16,24 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <gtk/gtk.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
-#include "ExternalProtocolService.h"
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#include <gtk/gtk.h>
 
 #include "nsString.h"
 #include "nsXPIDLString.h"
 #include "nsIMailtoUrl.h"
 #include "nsCOMPtr.h"
 #include "nsIURI.h"
+#include "nsCRT.h"
 
 extern "C" {
 #include "prefs_strings.h"
@@ -39,7 +48,9 @@ extern "C" {
 #include <libgnome/gnome-config.h>
 }
 
-void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL);
+#include "ExternalProtocolService.h"
+
+void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL, const char *url);
 
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GExternalProtocolService, nsIExternalProtocolService)
@@ -62,8 +73,8 @@ NS_IMETHODIMP GExternalProtocolService::
 {
 	*_retval = PR_FALSE;
 
-	nsCString cProtocolScheme = aProtocolScheme;
-	cProtocolScheme.Append("-show");
+	nsCAutoString cProtocolScheme(nsDependentCString(aProtocolScheme) +
+				      NS_LITERAL_CSTRING("-show"));
 
 	nsXPIDLCString key, value;
 	for (void *pI = gnome_config_init_iterator ("/Gnome/URL Handlers/") ;
@@ -71,7 +82,7 @@ NS_IMETHODIMP GExternalProtocolService::
 	     pI = gnome_config_iterator_next (pI, getter_Copies(key),
 					      getter_Copies(value)))
 	{
-		if (!nsCRT::strcasecmp(cProtocolScheme.get(), key.get()))
+		if (cProtocolScheme.Equals(key))
 		{
 			*_retval = PR_TRUE;
 			break;
@@ -84,7 +95,7 @@ NS_IMETHODIMP GExternalProtocolService::
 NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 {
 	gchar *mailer;
-	nsCString cSpec;
+	nsCAutoString cSpec;
 	aURL->GetSpec (cSpec);
 	nsXPIDLCString cScheme;
 	aURL->GetScheme (cScheme);
@@ -97,18 +108,18 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 	
 		if (mailer && strcmp (mailer, "Gnome") != 0)
 		{
-			OpenLegacyMailer (mailer, mailUrl);
+			OpenLegacyMailer (mailer, mailUrl, cSpec.get());
 			return NS_OK;
 		}
 	}
 
 	/* build the config key */
 
-	nsCString key = g_strconcat("/Gnome/URL Handlers/",
-				    cScheme.get(), "-show", NULL);
+	const nsCAutoString key(NS_LITERAL_CSTRING("/Gnome/URL Handlers/") +
+				cScheme + NS_LITERAL_CSTRING("-show"));
 
 	/* find it */
-	nsCString result = gnome_config_get_string(key.get());
+	nsCAutoString result(gnome_config_get_string(key.get()));
 	if (result.Length() != 0)
 	{
 		gnome_url_show(cSpec.get());
@@ -121,7 +132,7 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 	result = gnome_config_get_string("/Gnome/URL Handlers/default-show");
 
 		/* check there is a default */
-	if (result.Length() == 0)
+	if (result.IsEmpty())
 	{
 		/* throw the error */
 		gnome_error_dialog(_("Galeon cannot handle this protocol,\n"
@@ -152,6 +163,15 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
 	}
 }
 
+#if MOZILLA_SNAPSHOT > 12
+NS_IMETHODIMP GExternalProtocolService::IsExposedProtocol(const char *aProtocolScheme, PRBool *_retval)
+{
+	*_retval = PR_TRUE;
+
+	return NS_OK;
+}
+#endif
+
 /*
  * Open a legacy mailer according to preferences command
  *
@@ -171,9 +191,10 @@ NS_IMETHODIMP GExternalProtocolService::LoadUrl(nsIURI *aURL)
  * %s Subject
  * %r Reply
  * %o Follow up
+ * %u Unchanged URL
  * [ ] ignore all content if the part is null
  */
-void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL)
+void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL, const char *url)
 {
 	gchar *ToPart, *CcPart, *BccPart, *FromPart, 
 		*FollowUpToPart, *OrganizationPart, 
@@ -184,7 +205,8 @@ void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL)
 	gboolean forcePlainText;
 	
 	int length, i, j, subpos = -1;
-	char *copy, *command;
+	const char *copy;
+	char *command;
 
 	aURL->GetMessageContents (&ToPart, 
 				  &CcPart, 
@@ -222,7 +244,8 @@ void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL)
 			   (PriorityPart != NULL ? strlen (PriorityPart) : 0) +
 			   (NewsgroupPart != NULL ? strlen (NewsgroupPart) : 0) +
 			   (NewsHostPart != NULL ? strlen (NewsHostPart) : 0) +
-			   (BccPart != NULL ? strlen (BccPart) : 0));
+			   (BccPart != NULL ? strlen (BccPart) : 0) +
+			   (url != NULL ? strlen (url) : 0));
                                   
         /* parse the format string doing substitutions */
         for (i = 0, j = 0; i < length; i++)
@@ -285,6 +308,9 @@ void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL)
                         case 'b':
                                 copy = BccPart;
                                 break;
+                        case 'u':
+                                copy = url;
+                                break;
                         default:
                                 copy = NULL;
                                 break;
@@ -338,3 +364,4 @@ void OpenLegacyMailer(char *format, nsIMailtoUrl *aURL)
 	nsMemory::Free (NewsgroupPart);
 	nsMemory::Free (NewsHostPart);
 }
+
