@@ -37,7 +37,6 @@
 #include "dialog.h"
 #include "misc_string.h"
 #include "eel-gconf-extensions.h"
-#include "gfilepicker.h"
 
 #include <gtk/gtkmain.h>
 #include <gtk/gtkcheckbutton.h>
@@ -50,14 +49,10 @@
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-stock.h>
 
-#include "nsIFilePicker.h"
-
 #include "nsCRT.h"
 #include "nsCOMPtr.h"
-#include "nsIFactory.h"
 #include "nsISupportsArray.h"
 #include "nsIServiceManager.h"
-#include "nsXPComFactory.h"
 
 #include "nsString.h"
 #include "nsXPIDLString.h"
@@ -75,65 +70,8 @@
 
 #include <libgnome/gnome-util.h>
 
-class GFilePicker;
+#include "FilePicker.h"
 
-extern "C" {
-void filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aHandler);
-void filePicker_save_as_cancel_cb (GtkButton *button, GFilePicker *aHandler);
-gboolean filePicker_save_as_close_cb (GtkWidget *dialog, GdkEvent *event,
-				      GFilePicker *aHandler);
-void filePicker_save_content_cb (GtkToggleButton *button,
-				 GFilePicker *aHandler);
-}
-
-/* Header file */
-class GFilePicker : public nsIFilePicker
-{
-	friend void filePicker_save_as_ok_cb (GtkButton *button,
-					      GFilePicker *aHandler);
-	friend void filePicker_save_as_cancel_cb (GtkButton *button,
-						  GFilePicker *aHandler);
-	friend gboolean filePicker_save_as_close_cb (GtkWidget *dialog,
-						     GdkEvent *event,
-						     GFilePicker *aHandler);
-	friend void filePicker_save_content_cb (GtkToggleButton *button,
-						GFilePicker *aHandler);
-  public:
-	NS_DECL_ISUPPORTS
-	NS_DECL_NSIFILEPICKER
-
-	enum { returnOKSaveContent = 256 };
-
-	GFilePicker(PRBool showContentCheck = PR_FALSE, FileFormat *fileFormats = NULL);
-	virtual ~GFilePicker();
-	/* additional members */
-	NS_IMETHODIMP InitWithGtkWidget (GtkWidget *parentWidget, 
-					 const char *title,PRInt16 mode);
-	NS_IMETHODIMP SanityCheck (PRBool *retval);
-
-	PRInt16 mSelectedFileFormat;
-
-  private:
-	nsCOMPtr<nsIDOMWindowInternal> mParentInternal;
-	nsCOMPtr<nsIDOMWindow> mParent;
-	nsString mTitle;
-	PRInt16 mMode;
-	FileFormat *mFileFormats;
-
-	nsString mFilter;
-	nsString mDefaultString;
-	nsCOMPtr<nsILocalFile> mFile;
-	nsCOMPtr<nsILocalFile> mDisplayDirectory;
-
-	PRBool mFileSelectorClosed;
-	PRBool mFileSelectorOk;
-	PRBool mShowContentCheck;
-	PRBool mSaveContent;
-
-	GtkWidget *mParentWidget;	
-	GtkWidget *mFileSelector;
-	GtkWidget *mFormatChooser;
-};
 
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
@@ -260,6 +198,14 @@ NS_IMETHODIMP GFilePicker::GetFileURL(nsIFileURL * *aFileURL)
 	return NS_OK;
 }
 
+#if MOZILLA_SNAPSHOT > 2
+/* readonly attribute nsISimpleEnumerator files; */
+NS_IMETHODIMP GFilePicker::GetFiles(nsISimpleEnumerator * *aFiles)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
 /* short show (); */
 NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 {
@@ -281,8 +227,8 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 	nsCAutoString cDirName;
 	mDisplayDirectory->GetNativePath (cDirName);
 
-	nsCAutoString cFullPath = cDirName + NS_LITERAL_CSTRING("/") +
-				  nsDependentCString(cFileName ? cFileName : "");
+	const nsCAutoString cFullPath(cDirName + NS_LITERAL_CSTRING("/") +
+				      nsDependentCString(cFileName ? cFileName : ""));
 	g_free (cFileName);
 
 	gtk_file_selection_set_filename (GTK_FILE_SELECTION(mFileSelector),
@@ -421,20 +367,28 @@ NS_IMETHODIMP GFilePicker::SanityCheck (PRBool *retval)
 {
 	*retval = PR_TRUE;
 
+	nsresult rv;
 	PRBool dirExists, fileExists = PR_TRUE;
 
 	if (mDisplayDirectory)
 	{
-		mDisplayDirectory->Exists (&dirExists);
+		rv = mDisplayDirectory->Exists (&dirExists);
+		g_return_val_if_fail (NS_SUCCEEDED(rv), rv);
 	}
 	else
 	{
 		dirExists = PR_FALSE;
 	}
 
-	if (mMode == nsIFilePicker::modeOpen)
+	if (mMode != nsIFilePicker::modeGetFolder)
 	{
-		mFile->Exists (&fileExists);
+		rv = mFile->Exists (&fileExists);
+		g_return_val_if_fail (NS_SUCCEEDED(rv), rv);
+	}
+	
+	if (mMode == nsIFilePicker::modeSave && !fileExists)
+	{
+		return NS_OK;
 	}
 	
 	if (!dirExists || !fileExists)
@@ -458,13 +412,15 @@ NS_IMETHODIMP GFilePicker::SanityCheck (PRBool *retval)
 	char *errorText;
 	if (mMode == nsIFilePicker::modeGetFolder)
 	{
-		mDisplayDirectory->IsDirectory (&correctType);
+		rv = mDisplayDirectory->IsDirectory (&correctType);
+		g_return_val_if_fail (NS_SUCCEEDED(rv), rv);
 		errorText = g_strdup (_("A file was selected when a "
 					"folder was expected."));
 	}
 	else
 	{
-		mFile->IsFile (&correctType);
+		rv = mFile->IsFile (&correctType);
+		g_return_val_if_fail (NS_SUCCEEDED(rv), rv);
 		errorText = g_strdup (_("A folder was selected when a "
 				        "file was expected."));
 	}
@@ -493,30 +449,6 @@ NS_IMETHODIMP GFilePicker::SanityCheck (PRBool *retval)
 // begin local private methods impl
 ////////////////////////////////////////////////////////////////////////////////
 
-//------------------------------------------------------------------------------
-
-NS_DEF_FACTORY (GFilePicker, GFilePicker);
-
-/**
- * NS_NewFilePickerFactory:
- */ 
-nsresult NS_NewFilePickerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGFilePickerFactory *result = new nsGFilePickerFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // begin FileSelector callbacks.
 ////////////////////////////////////////////////////////////////////////////////
@@ -703,7 +635,7 @@ show_file_picker (GtkWidget *parentWidget, const char* title,
 			g_free (*ret_fullpath);
 		nsCOMPtr<nsILocalFile> aFile;
 		aFilePicker->GetFile (getter_AddRefs(aFile));
-		nsCString tempFullPathStr;
+		nsCAutoString tempFullPathStr;
 		aFile->GetNativePath (tempFullPathStr);
 		*ret_fullpath = g_strdup (tempFullPathStr.get());
 		delete aFilePicker;
