@@ -25,8 +25,9 @@
 #include "PrintProgressListener.h"
 
 #include <gtkmozembed_internal.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-
+#endif
 #include "nsIContentViewer.h"
 #include "nsIPermissionManager.h"
 #include "nsIGlobalHistory.h"
@@ -48,8 +49,6 @@
 #include "nsIScriptContext.h"
 
 #include "nsIDOMWindowInternal.h"
-#include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIFocusController.h"
 #include "nsIWebBrowserPersist.h"
@@ -69,7 +68,11 @@
 #include "nsIDocumentObserver.h"
 #include "nsCWebBrowser.h"
 #include "nsReadableUtils.h"
+#if MOZILLA_SNAPSHOT > 7
+#include "nsIDOMNSDocument.h"
+#else
 #include "nsIDOMNSHTMLDocument.h"
+#endif
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
@@ -79,6 +82,15 @@
 #include "caps/nsIPrincipal.h"
 #include "nsIDeviceContext.h"
 #include "nsIPresContext.h"
+#if MOZILLA_SNAPSHOT > 2
+#include "nsITypeAheadFind.h"
+#include "nsSupportsPrimitives.h"
+#endif
+
+#if MOZILLA_SNAPSHOT > 9
+//Mapping table for permission types in mozilla.cpp
+extern const char** gPermType;
+#endif
 
 GaleonWrapper::GaleonWrapper ()
 {
@@ -108,18 +120,6 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 	nsCOMPtr<nsIDocShellHistory> dsHistory = do_QueryInterface (DocShell);
 	if (!dsHistory) return NS_ERROR_FAILURE;
 
-	static NS_DEFINE_CID(kGlobalHistoryCID, GALEON_GLOBALHISTORY_CID);
-
-	nsCOMPtr<nsIFactory> GHFactory;
-	result = NS_NewGlobalHistoryFactory(getter_AddRefs(GHFactory));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	result = nsComponentManager::RegisterFactory(kGlobalHistoryCID,
-						     "Global history",
-						     NS_GLOBALHISTORY_CONTRACTID,
-						     GHFactory,
-						     PR_TRUE);
-
 	nsCOMPtr<nsIGlobalHistory> inst =  
 		do_GetService(NS_GLOBALHISTORY_CONTRACTID, &result);
 	return dsHistory->SetGlobalHistory(inst);
@@ -177,53 +177,63 @@ nsresult GaleonWrapper::Print (nsIPrintSettings *options, PRBool preview)
 	if (!preview)
 	{
 		GPrintListener *listener = new GPrintListener();
-#if MOZILLA_VERSION > VERSION3(0,9,8)
 		result = print->Print (options, listener);
-#else
-		result = print->Print (DOMWindow, options, listener);
-#endif
 	}
 	else
 	{
-		char *filename;
-
-		filename = g_strconcat(g_get_tmp_dir(),"/galeon-print-XXXXXX", NULL);
-		/* get a name for the temporary file */
-		int res;
-		res = mkstemp (filename);
-		if (res == -1)
-			return NS_ERROR_FAILURE;
-		close (res);
-		mozilla_delete_temp_file_on_exit (filename);
-
-		PRUnichar *uc_filename = mozilla_locale_to_unicode( filename );
-		options->SetToFileName ( uc_filename );
-		g_free( uc_filename );
-		options->SetPrintToFile (TRUE);
-
-		GPrintListener *listener =
-			new GPrintListener(filename);
-#if MOZILLA_VERSION > VERSION3(0,9,8)
-		result = print->Print (options, listener);
-#else
-		result = print->Print (DOMWindow, options, listener);
-#endif
-		g_free (filename);
+		result = print->PrintPreview(options, nsnull, nsnull);
 	}
 
 	return result;
 }
 
+nsresult GaleonWrapper::PrintPreviewClose (void)
+{
+	nsresult rv;
+	PRBool isPreview = PR_FALSE;
+
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &rv));
+	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
+
+	rv = print->GetDoingPrintPreview(&isPreview);
+	if (isPreview == PR_TRUE)
+	{
+		rv = print->ExitPrintPreview();
+	}
+
+	return rv;
+}
+
+nsresult GaleonWrapper::PrintPreviewNumPages (int *numPages)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &rv));
+	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
+
+	rv = print->GetPrintPreviewNumPages(numPages);
+	return rv;
+}
+
+nsresult GaleonWrapper::PrintPreviewNavigate(PRInt16 navType, PRInt32 pageNum)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &rv));
+	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
+
+	rv = print->PrintPreviewNavigate(navType, pageNum);
+	return rv;
+}
+
 nsresult GaleonWrapper::GetPrintSettings (nsIPrintSettings **options)
 {
 	nsresult result;
 	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &result));
+
 	if (NS_FAILED(result) || !print) return NS_ERROR_FAILURE;
-#if MOZILLA_VERSION > VERSION3(0,9,8)
+
 	return print->GetGlobalPrintSettings(options);
-#else
-	return print->GetPrintSettings(options);
-#endif
 }
 
 nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
@@ -323,10 +333,11 @@ nsresult GaleonWrapper::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
 
 	nsCOMPtr<nsIPresContext> PresContext;
 	result = DocShell->GetPresContext (getter_AddRefs(PresContext));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+	if (NS_FAILED(result) || !PresContext) return NS_ERROR_FAILURE;
 					
 	nsCOMPtr<nsIDeviceContext> DeviceContext;
 	result = PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
+	if (NS_FAILED(result) || !DeviceContext) return NS_ERROR_FAILURE;
 
 	return DeviceContext->SetTextZoom (aZoom);
 }
@@ -409,7 +420,7 @@ nsresult GaleonWrapper::SaveURI (const char *url, char *filename, DownloadAction
 {
         nsresult rv;
 
-	nsString s;
+	nsAutoString s;
 	s.AssignWithConversion(url);
 	nsCOMPtr<nsIURI> linkURI;
 	rv = NS_NewURI(getter_AddRefs(linkURI), s);
@@ -420,7 +431,8 @@ nsresult GaleonWrapper::SaveURI (const char *url, char *filename, DownloadAction
 	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsILocalFile> file;
-	NS_NewLocalFile(filename, PR_TRUE, getter_AddRefs(file)); 
+	NS_NewNativeLocalFile(nsDependentCString(filename),
+			      PR_TRUE, getter_AddRefs(file)); 
 	if (NS_FAILED(rv) || !file) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMWindow> parent;
@@ -452,7 +464,11 @@ nsresult GaleonWrapper::SaveURI (const char *url, char *filename, DownloadAction
 			persist->SetPersistFlags (nsIWebBrowserPersist::PERSIST_FLAGS_NONE);
 		break;
 	}
+#if MOZILLA_SNAPSHOT > 4
+	return persist->SaveURI(linkURI, nsnull, nsnull, nsnull, nsnull, file);
+#else
 	return persist->SaveURI (linkURI, nsnull, file);
+#endif
 }
 
 nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
@@ -472,12 +488,14 @@ nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
 	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsILocalFile> file;
-	rv = NS_NewLocalFile(filename, PR_TRUE, getter_AddRefs(file)); 
+	NS_NewNativeLocalFile(nsDependentCString(filename),
+			      PR_TRUE, getter_AddRefs(file)); 
 	if (NS_FAILED(rv) || !file) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsILocalFile> path;
 	if (datapath)
-		NS_NewLocalFile(datapath, PR_TRUE, getter_AddRefs(path));
+		NS_NewNativeLocalFile(nsDependentCString(datapath),
+				      PR_TRUE, getter_AddRefs(path));
 	else
 		path = nsnull;
 
@@ -489,8 +507,13 @@ nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
 		if (NS_FAILED(rv) || !aDocument) return NS_ERROR_FAILURE;
 
 		nsCOMPtr<nsIURI> aURI;
+#if MOZILLA_SNAPSHOT > 11
+		aURI = aDocument->GetDocumentURL();
+		if (!aURI) return NS_ERROR_FAILURE;
+#else
 		rv = aDocument->GetDocumentURL (getter_AddRefs(aURI));
 		if (NS_FAILED(rv) || !aURI) return NS_ERROR_FAILURE;
+#endif
 
 		nsCOMPtr<nsIDOMWindow> parent;
 		mWebBrowser->GetContentDOMWindow (getter_AddRefs(parent));
@@ -559,13 +582,42 @@ nsresult GaleonWrapper::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
 	return NS_OK;
 }
 
-nsresult GaleonWrapper::Find (const PRUnichar *search_string, 
+nsresult GaleonWrapper::Find (const PRUnichar *search_string,
+			      PRBool interactive,
 			      PRBool matchcase, PRBool search_backwards,
 			      PRBool search_wrap_around,
 			      PRBool search_for_entire_word,
 			      PRBool search_in_frames,
 			      PRBool *didFind)
 {
+#if MOZILLA_SNAPSHOT > 2
+	if (!interactive)
+	{
+		nsresult rv;
+		nsCOMPtr<nsITypeAheadFind> tAFinder
+			(do_GetService(NS_TYPEAHEADFIND_CONTRACTID, &rv));
+		if (NS_SUCCEEDED(rv))
+		{
+			nsCOMPtr<nsIDOMWindow> aFocusedWindow;
+			rv = GetFocusedDOMWindow(getter_AddRefs(aFocusedWindow));
+			if (NS_SUCCEEDED(rv))
+			{
+				nsSupportsInterfacePointerImpl windowPtr;
+				windowPtr.SetData(aFocusedWindow);
+
+				tAFinder->FindNext(search_backwards, &windowPtr);
+
+				nsCOMPtr<nsISupports> retValue;
+				rv = windowPtr.GetData(getter_AddRefs(retValue));
+				if (NS_SUCCEEDED(rv) && !retValue)
+				{
+					*didFind = PR_TRUE;
+					return NS_OK;
+				}
+			}
+		}
+	}
+#endif
 	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
 
 	finder->SetSearchString (search_string);
@@ -577,14 +629,12 @@ nsresult GaleonWrapper::Find (const PRUnichar *search_string,
 	return finder->FindNext(didFind);
 }
 
-nsresult GaleonWrapper::ReloadDocument ()
+nsresult GaleonWrapper::GetWebNavigation(nsIWebNavigation **aWebNavigation)
 {
 	nsresult result;
 
-	nsCOMPtr<nsIWebBrowserFocus> focus(do_GetInterface(mWebBrowser));
-
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = focus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
 	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIScriptGlobalObject> scriptGlobal = do_QueryInterface(DOMWindow);
@@ -597,6 +647,18 @@ nsresult GaleonWrapper::ReloadDocument ()
 	nsCOMPtr<nsIWebNavigation> wn = do_QueryInterface (docshell, &result);
 	if (!wn || !NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
 
+	NS_IF_ADDREF(*aWebNavigation = wn);
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::ReloadDocument ()
+{
+	nsresult result;
+
+	nsCOMPtr<nsIWebNavigation> wn;
+	result = GetWebNavigation(getter_AddRefs(wn));
+	if (!wn || !NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
+
 	result = wn->Reload (nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
 			     nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY);
 	if (!NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
@@ -604,6 +666,35 @@ nsresult GaleonWrapper::ReloadDocument ()
 	return NS_OK;
 }
 
+nsresult GaleonWrapper::LoadDocument(nsISupports *aPageDescriptor,
+				     PRUint32 aDisplayType)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIWebNavigation> wn;
+	rv = GetWebNavigation(getter_AddRefs(wn));
+	if (!wn || !NS_SUCCEEDED(rv)) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIWebPageDescriptor> wpd = do_QueryInterface(wn, &rv);
+	if (!wpd || !NS_SUCCEEDED(rv)) return NS_ERROR_FAILURE;
+
+	return wpd->LoadPage(aPageDescriptor, aDisplayType);
+}
+
+nsresult GaleonWrapper::GetPageDescriptor(nsISupports **aPageDescriptor)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIWebNavigation> wn;
+	rv = GetWebNavigation(getter_AddRefs(wn));
+	if (!wn || !NS_SUCCEEDED(rv)) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIWebPageDescriptor> wpd = do_QueryInterface(wn, &rv);
+	if (!wpd || !NS_SUCCEEDED(rv)) return NS_ERROR_FAILURE;
+
+	return wpd->GetCurrentDescriptor(aPageDescriptor);
+}
+
 nsresult GaleonWrapper::GetMainDocumentUrl (nsCString &url)
 {
 	nsresult result;
@@ -617,7 +708,11 @@ nsresult GaleonWrapper::GetMainDocumentUrl (nsCString &url)
 	if(!doc) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+	uri = doc->GetDocumentURL();
+#else
 	doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
 	return uri->GetSpec (url);
 }
@@ -635,7 +730,11 @@ nsresult GaleonWrapper::GetDocumentUrl (nsCString &url)
         if(!doc) return NS_ERROR_FAILURE;
 
         nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+	uri = doc->GetDocumentURL();
+#else
         doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
         uri->GetSpec (url);
 
@@ -648,16 +747,28 @@ nsresult GaleonWrapper::GetDocumentTitle (char **title)
 
 	nsCOMPtr<nsIDOMDocument> aDOMDocument;
 
-	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
+	result = GetDOMDocument(getter_AddRefs(aDOMDocument));
 	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
-	if(!doc) return NS_ERROR_FAILURE;
+	if (!doc) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT > 11
+	nsString docTitle(doc->GetDocumentTitle());
+
+	*title = mozilla_unicode_to_locale(docTitle.get());
+#elif MOZILLA_SNAPSHOT > 6
+	nsString docTitle;
+	result = doc->GetDocumentTitle(docTitle);
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+
+	*title = mozilla_unicode_to_locale(docTitle.get());
+#else
 	const nsString* t;
 	t = doc->GetDocumentTitle();
 
-	*title = mozilla_unicode_to_locale (t->get());
+	*title = mozilla_unicode_to_locale(t->get());
+#endif
 
 	return NS_OK;
 }
@@ -724,27 +835,45 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 }
 
 
-nsresult GaleonWrapper::SetSitePermission (gboolean block, PRInt32 type)
+nsresult GaleonWrapper::SetSitePermission (PRBool allow, PRUint32 type)
 {
 	nsresult result;
 
 	nsCOMPtr<nsIDOMDocument> DOMDocument;
-	result = GetDOMDocument (getter_AddRefs(DOMDocument));
+	result = GetDOMDocument(getter_AddRefs(DOMDocument));
 	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface (DOMDocument);
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
 	if (!doc) return NS_ERROR_FAILURE;
 	
 	nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+	uri = doc->GetDocumentURL();
+#else
 	doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
-	nsCString url;
-	uri->GetSpec (url);
+	nsCAutoString url;
+	uri->GetSpec(url);
 	
 	nsCOMPtr<nsIPermissionManager> permissionManager =
-			do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
-	
-	result = permissionManager->Add (url.get(), block ? PR_TRUE : PR_FALSE, type);
+			do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
+
+#if MOZILLA_SNAPSHOT > 9
+	result = permissionManager->Add(uri, gPermType[type],
+					allow ?
+					static_cast<PRUint32>(nsIPermissionManager::ALLOW_ACTION) :
+					static_cast<PRUint32>(nsIPermissionManager::DENY_ACTION));
+#elif MOZILLA_SNAPSHOT > 6
+	result = permissionManager->Add(uri, type,
+					allow ?
+					static_cast<PRUint32>(nsIPermissionManager::ALLOW_ACTION) :
+					static_cast<PRUint32>(nsIPermissionManager::DENY_ACTION));
+#elif MOZILLA_SNAPSHOT > 2
+	result = permissionManager->Add (url, allow, type);
+#else
+	result = permissionManager->Add (url.get(), allow, type);
+#endif
 
 	return result;
 }
@@ -765,9 +894,13 @@ nsresult GaleonWrapper::ForceCharacterSet (char *charset)
 								  &result);
 	if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT > 9
+	result = mdv->SetForceCharacterSet(nsDependentCString(charset));
+#else
 	nsAutoString charset_str;
 	charset_str.AssignWithConversion (charset);
 	result = mdv->SetForceCharacterSet (ToNewUnicode(charset_str));
+#endif
 
 	return result;
 }
@@ -854,7 +987,7 @@ nsresult GaleonWrapper::GetFaviconURL (nsCString &favicon_url)
 			if (NS_FAILED (result) || value.IsEmpty())
 				return NS_ERROR_FAILURE;
 			
-			nsCString link;
+			nsCAutoString link;
 			link.AssignWithConversion(value);
 
 			nsCOMPtr<nsIDocument> doc = 
@@ -862,8 +995,13 @@ nsresult GaleonWrapper::GetFaviconURL (nsCString &favicon_url)
 			if(!doc) return NS_ERROR_FAILURE;
 			
 			nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+			uri = doc->GetDocumentURL();
+			if (!uri) return NS_ERROR_FAILURE;
+#else
 			result = doc->GetDocumentURL(getter_AddRefs(uri));
 			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+#endif
 
 			result = uri->Resolve (link, favicon_url);
 			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
@@ -896,7 +1034,6 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 					       nsIStyleSheet **return_sheet)
 {
 	nsresult result;
-	PRBool completed;
 
 	/* catch necessary stuff */
 	nsCOMPtr<nsIDocShell> DocShell;
@@ -918,7 +1055,7 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	/* load sheet */
-	nsString s;
+	nsAutoString s;
 	s.AssignWithConversion(css);
 	
 	nsCOMPtr<nsIURI> uri;
@@ -926,7 +1063,12 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	nsICSSStyleSheet *sheet;
+#if MOZILLA_SNAPSHOT > 3
+	result = loader->LoadAgentSheet(uri, &sheet);
+#else
+	PRBool completed;
 	result = loader->LoadAgentSheet(uri, sheet, completed, nsnull);
+#endif
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	/* catch stylesheet stuff and apply by appending it as a override
@@ -1006,7 +1148,7 @@ nsresult GaleonWrapper::GetLinkInterfaceItems (GList **list)
 	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
 
 	/* get list of link elements*/
-	nsString strname;
+	nsAutoString strname;
 	strname.AssignWithConversion("LINK");
 
 	nsCOMPtr<nsIDOMNodeList> aLinks;
@@ -1068,7 +1210,7 @@ nsresult GaleonWrapper::GetLinkInterfaceItems (GList **list)
 					continue;
 				}
 
-				nsCString link;
+				nsCAutoString link;
 				link.AssignWithConversion(value);
 
 				/* resolve uri */
@@ -1077,9 +1219,13 @@ nsresult GaleonWrapper::GetLinkInterfaceItems (GList **list)
 				if(!doc) return NS_ERROR_FAILURE;
 			
 				nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+				uri = doc->GetDocumentURL();
+#else
 				doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
-				nsCString href;
+				nsCAutoString href;
 				result = uri->Resolve (link, href);
 				if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 				lti->href = g_strdup (href.get());
@@ -1160,18 +1306,15 @@ nsresult GaleonWrapper::ScrollDown (void)
 
 nsresult GaleonWrapper::ScrollLeft (void)
 {
-	nsresult result;
-
-	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
-	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
-
-	DOMWindow->ScrollBy(-16, 0);
-	
-	return NS_OK;
+	return ScrollBy (-16, 0);
 }
 
 nsresult GaleonWrapper::ScrollRight (void)
+{
+	return ScrollBy (16, 0);
+}
+
+nsresult GaleonWrapper::ScrollBy (gint x, gint y)
 {
 	nsresult result;
 
@@ -1179,7 +1322,7 @@ nsresult GaleonWrapper::ScrollRight (void)
 	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
 	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
 
-	DOMWindow->ScrollBy(16, 0);
+	DOMWindow->ScrollBy (x, y);
 	
 	return NS_OK;
 }
@@ -1193,7 +1336,11 @@ nsresult GaleonWrapper::GetLastModified (gchar **ret)
 	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
 	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT > 7
+	nsCOMPtr<nsIDOMNSDocument> doc = do_QueryInterface(aDOMDocument);
+#else
 	nsCOMPtr<nsIDOMNSHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+#endif
 	if(!doc) return NS_ERROR_FAILURE;
 
 	nsAutoString value;
@@ -1320,7 +1467,7 @@ nsresult GaleonWrapper::GetForms (GList **ret)
 		result = form->GetAction (tmp);
 		if (NS_SUCCEEDED(result))
 		{
-			nsCString c, s;
+			nsCAutoString c, s;
 			c.AssignWithConversion(tmp);
 
 			nsCOMPtr<nsIDocument> doc = 
@@ -1328,7 +1475,11 @@ nsresult GaleonWrapper::GetForms (GList **ret)
 			if(!doc) return NS_ERROR_FAILURE;
 			
 			nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+			uri = doc->GetDocumentURL();
+#else
 			doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
 			result = uri->Resolve (c, s);
 
@@ -1368,7 +1519,7 @@ nsresult GaleonWrapper::GetLinks (GList **ret)
 	/* first, get a list of <link> elements */
 	PRUint32 links_count;
 
-	nsString strname;
+	nsAutoString strname;
 	strname.AssignWithConversion("LINK");
 
 	nsCOMPtr<nsIDOMNodeList> aLinks;
@@ -1395,7 +1546,7 @@ nsresult GaleonWrapper::GetLinks (GList **ret)
 		linkElement->GetAttribute (attr, value);
 		if (value.IsEmpty()) continue;
 
-		nsCString link;
+		nsCAutoString link;
 		link.AssignWithConversion(value);
 
 		if (link.IsEmpty()) continue;
@@ -1405,9 +1556,13 @@ nsresult GaleonWrapper::GetLinks (GList **ret)
 		if(!doc) continue;
 		
 		nsCOMPtr<nsIURI> uri;
+#if MOZILLA_SNAPSHOT > 11
+		uri = doc->GetDocumentURL();
+#else
 		doc->GetDocumentURL(getter_AddRefs(uri));
+#endif
 
-		nsCString tmp;
+		nsCAutoString tmp;
 		result = uri->Resolve (link, tmp);
 		
 		LinkListItem *i = g_new0 (LinkListItem, 1);
@@ -1550,7 +1705,7 @@ nsresult GaleonWrapper::EvaluateJS (char *script)
 	context->SetProcessingScriptTag(PR_TRUE);
 
 	PRBool isUndefined;
-	nsString ret;
+	nsAutoString ret;
 	nsAutoString aScript;
 	aScript.AssignWithConversion (script);
 	context->EvaluateString(aScript, nsnull, nsnull, nsnull,
