@@ -21,6 +21,7 @@
 
 #include "nsIDocShell.h"
 #include "nsIWebNavigation.h"
+#include "nsIWebPageDescriptor.h"
 #include "nsISHistory.h"
 #include "nsIWebBrowser.h"
 #include "nsIWebProgressListener.h"
@@ -53,9 +54,13 @@ public:
 
 	nsresult Print (nsIPrintSettings *options, PRBool preview);
 	nsresult GetPrintSettings (nsIPrintSettings * *options);
+	nsresult PrintPreviewClose (void);
+	nsresult PrintPreviewNumPages (int *numPages);
+	nsresult PrintPreviewNavigate(PRInt16 navType, PRInt32 pageNum);
 
-	nsresult Find (const PRUnichar *search_string, PRBool matchcase,
-		       PRBool search_backwards, PRBool search_wrap_around,
+	nsresult Find (const PRUnichar *search_string, PRBool interactive,
+		       PRBool matchcase, PRBool search_backwards,
+		       PRBool search_wrap_around, 
 		       PRBool search_for_entire_word, PRBool search_in_frames,
 		       PRBool *didFind);
 
@@ -64,6 +69,8 @@ public:
 	nsresult GetDocumentTitle (char **title);
 
 	nsresult ReloadDocument ();
+	nsresult LoadDocument(nsISupports *aPageDescriptor, PRUint32 aDisplayType);
+	nsresult GetPageDescriptor(nsISupports **aPageDescriptor);
 
 	nsresult GetSHInfo (PRInt32 *count, PRInt32 *index);
 	nsresult GetSHTitleAtIndex (PRInt32 index, PRUnichar **title);
@@ -73,7 +80,7 @@ public:
 
 	nsresult GoToHistoryIndex (PRInt16 index);
 
-	nsresult SetSitePermission( PRBool permission, PRInt32 type );
+	nsresult SetSitePermission( PRBool permission, PRUint32 type );
 
 	nsresult ForceCharacterSet (char *charset);
 
@@ -114,6 +121,7 @@ public:
 	nsresult ScrollDown (void);
 	nsresult ScrollLeft (void);
 	nsresult ScrollRight (void);
+ 	nsresult ScrollBy (gint x, gint y);
 
 	nsresult GetLastModified (gchar **ret);
 	nsresult GetImages (GList **ret);
@@ -142,6 +150,7 @@ private:
 	nsresult GetDOMDocument (nsIDOMDocument **aDOMDocument);
 	nsresult GetSHistory (nsISHistory **aSHistory);
 	nsresult GetPIDOMWindow(nsPIDOMWindow **aPIWin);
+	nsresult GetWebNavigation(nsIWebNavigation **aWebNavigation);
 };
 
 #endif
