@@ -16,8 +16,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libgnome/gnome-help.h>
 
+#include "nsReadableUtils.h"
+
 #include "GnomeHelpProtocolHandler.h"
 
 /* Implementation file */
@@ -30,22 +36,21 @@ NS_IMETHODIMP GGnomeHelpProtocolHandler::NewChannel(nsIURI *aURI,
 	nsresult rv;
 	mURI = aURI;
 
-	nsCString host;
+	nsCAutoString host;
 	rv = aURI->GetHost (host);
 	if (NS_FAILED(rv)) return rv;
 
-	nsCString path;
+	nsCAutoString path;
 	rv = aURI->GetPath (path);
 	if (NS_FAILED(rv)) return rv;
 
-	if (strstr(path.get(), ".sgml"))
+	if (FindInReadable(NS_LITERAL_CSTRING(".sgml"), path))
 	{
 		mDocName = path;
 	}
 	else
 	{
-		nsCString fileName = host;
-		fileName += ".sgml";
+		const nsCAutoString fileName(host + NS_LITERAL_CSTRING(".sgml"));
 		char *helpPath = gnome_help_file_find_file (
 					const_cast<char *>(host.get()),
 					const_cast<char *>(fileName.get()));
