@@ -20,13 +20,14 @@
 #define __GJSConsoleService_h
 
 // {35807600-35bd-11d5-bb6f-b9f2e9fee03c}
-#define G_JSCONSOLESERVICE_CID \
+#define G_JS_CONSOLESERVICE_CLASSNAME \
+ "Galeon's Javascript Console Implementation"
+#define G_JS_CONSOLESERVICE_CID \
  { 0x35807600, 0x35bd, 0x11d5, {0xbb, 0x6f, 0xb9, 0xf2, 0xe9, 0xfe, 0xe0, 0x3c}}
 #define NS_JSCONSOLESERVICE_CONTRACTID \
  "@mozilla.org/embedcomp/jsconsole-service;1"
 
 #include "nsIJSConsoleService.h"
-#include "nsIFactory.h"
 
 class nsIDOMWindow;
 
@@ -42,7 +43,5 @@ public:
 
 };
 
-extern nsresult NS_NewJSConsoleServiceFactory(nsIFactory** aFactory);
-
 #endif
 
