@@ -16,43 +16,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "nsIFactory.h"
-#include "nsXPComFactory.h"
-#include "nsXPIDLString.h"
 #include "nsIURI.h"
 #include "nsIMailtoUrl.h"
 #include "nsNetUtil.h"
 #include "nsIExternalProtocolService.h"
 #include "nsCExternalHandlerService.h"
 
-#include "BaseProtocolContentHandler.h"
 #include "gUrlCID.h"
+#include "MailtoProtocolHandler.h"
 
 static NS_DEFINE_CID(kMailtoURLCID, NS_MAILTOURL_CID);
 
-class GMailtoProtocolHandler : public GBaseProtocolContentHandler
-{
-  public:
-	NS_DECL_ISUPPORTS
-	GMailtoProtocolHandler() : GBaseProtocolContentHandler("mailto")
-				   {NS_INIT_ISUPPORTS();};
-	virtual ~GMailtoProtocolHandler() {};
-	/* additional members */
-	NS_IMETHODIMP NewURI(const nsACString &aSpec, const char *aOrignCharset,
-			     nsIURI *aBaseURI, nsIURI **_retval);
-	NS_IMETHODIMP HandleContent (const char * aContentType,
-				     const char * aCommand,
-				     nsISupports * aWindowContext,
-				     nsIRequest *aRequest);
-  private:
-  	nsXPIDLCString	mToPart, mCcPart, mBccPart, mFromPart, mFollowUpToPart,
-  			mOrganisationPart, mReplyToPart, mSubjectPart, mBodyPart,
-  			mHtmlPart, mReferencePart, mAttachmentPart, mPriortyPart,
-  			mNewsgroupPart, mNewsHostPart;
-  	PRBool		mForcePlainText;
-  	
-};
-
 /* Implementation file */
 NS_IMPL_ISUPPORTS2 (GMailtoProtocolHandler, nsIProtocolHandler, nsIContentHandler)
 
@@ -90,7 +64,7 @@ NS_IMETHODIMP GMailtoProtocolHandler::HandleContent (
 		return NS_ERROR_NULL_POINTER;
   	// First of all, get the content type and make sure it is a 
   	// content type we know how to handle!
-	if (nsCRT::strcasecmp(aContentType, mMimeType.get()) == 0)
+	if (mMimeType.Equals(aContentType))
 	{
 		nsCOMPtr<nsIChannel> aChannel = do_QueryInterface(aRequest);
 		if(!aChannel) return NS_ERROR_FAILURE;
@@ -114,26 +88,3 @@ NS_IMETHODIMP GMailtoProtocolHandler::HandleContent (
 	}
 	return rv;
 }
-
-NS_DEF_FACTORY (GMailtoProtocolHandler, GMailtoProtocolHandler);
-
-/**
- * NS_NewMailtoHandlerFactory:
- */ 
-nsresult NS_NewMailtoHandlerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGMailtoProtocolHandlerFactory *result = 
-					new nsGMailtoProtocolHandlerFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
