@@ -21,22 +21,24 @@
 #include "misc_general.h"
 #include "eel-gconf-extensions.h"
 
+#ifdef HAVE_STDLIB_H 
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 #include <libgnome/gnome-i18n.h>
 
-#include "nsIAboutModule.h"
 #include "nsIIOService.h"
 #include "nsIServiceManager.h"
-#include "nsIFactory.h"
-#include "nsXPComFactory.h"
 #include "nsCOMPtr.h"
-#include "nsIURI.h"
 #include "nsNetCID.h"
 #include "nsIStorageStream.h"
 #include "nsNetUtil.h"
 #include "mozilla.h"
 
+#include "MyportalProtocolHandler.h"
+
 /* set this to use a file in /tmp rather than generating on the fly  */
 #define USE_INTERMEDIATE_FILE 0
 #define MAX_LINK_NAME_LENGTH 40
@@ -69,22 +71,6 @@ static PRUint32 bytesWriten;
 
 static NS_DEFINE_CID(kSimpleURICID,     NS_SIMPLEURI_CID);
 
-class GMyportalProtocolHandler : public nsIProtocolHandler,
-				 public nsIAboutModule
-{
-  public:
-	NS_DECL_ISUPPORTS
-	NS_DECL_NSIPROTOCOLHANDLER
-
-	GMyportalProtocolHandler (void);
-	virtual ~GMyportalProtocolHandler();
-
-	/* additional members */
-	NS_METHOD CreateMyportalPage (const nsCString &path);
-	nsCOMPtr<nsIChannel> mChannel;
-	nsCOMPtr<nsIURI> mURI;
-};
-
 /* Implementation file */
 NS_IMPL_ISUPPORTS2 (GMyportalProtocolHandler, nsIProtocolHandler, nsIAboutModule)
 
@@ -158,10 +144,10 @@ NS_IMETHODIMP GMyportalProtocolHandler::NewChannel(nsIURI *aURI,
 	rv =  aURI->SchemeIs ("about", &isabout);
 	if (NS_FAILED(rv)) return rv;
 	if (isabout)
-		rv = CreateMyportalPage ("/");
+		rv = CreateMyportalPage (NS_LITERAL_CSTRING("/"));
 	else
 	{
-		nsCString path;
+		nsCAutoString path;
 		rv = aURI->GetPath (path);
 		if (NS_FAILED(rv)) return rv;
 		rv = CreateMyportalPage (path);
@@ -178,7 +164,7 @@ NS_IMETHODIMP GMyportalProtocolHandler::AllowPort(PRInt32 port, const char *sche
 	return NS_OK;
 }
 
-NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsCString &path)
+NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsACString &path)
 {
 	nsresult rv;
 
@@ -231,7 +217,7 @@ NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsCString &path)
 	RENDER ("<a href=\"" GALEON_HOMEPAGE_URL "\">"
 	       "<img class=\"logo\" src=\"file:" SHARE_DIR
 	       "/logo.png\" alt=\"\"></a>");
-	render_from_path (stream, path.get());
+	render_from_path (stream, PromiseFlatCString(path).get());
 	RENDER ("</body></html>\n");
 	g_free(myportal_utfstr);
 
@@ -248,45 +234,29 @@ NS_METHOD GMyportalProtocolHandler::CreateMyportalPage (const nsCString &path)
 
 	g_free (myportalURI);
 #else
-	nsCOMPtr<nsIInputStream> iStream;
-	PRUint32 size;  
-    
+        nsCOMPtr<nsIInputStream> iStream;
+        rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
+        if (NS_FAILED(rv)) return rv;
+
+#if MOZILLA_SNAPSHOT > 4
+	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
+				      iStream, NS_LITERAL_CSTRING("text/html"),
+				      NS_LITERAL_CSTRING("utf-8"));
+#else
+	PRUint32 size;
 	rv = sStream->GetLength(&size);
 	if (NS_FAILED(rv)) return rv;
 
-	rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
-	if (NS_FAILED(rv)) return rv;
-
 	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
-				      iStream, "text/html", size);
+				      iStream, NS_LITERAL_CSTRING("text/html"),
+				      NS_LITERAL_CSTRING("utf-8"), size);
+#endif
 	if (NS_FAILED(rv)) return rv;
 #endif
 	g_free (stylesheet_filename);
 	return rv;
 }
 
-NS_DEF_FACTORY (GMyportalProtocolHandler, GMyportalProtocolHandler);
-
-/**
- * NS_NewMyportalProtocolHandlerFactory:
- */ 
-nsresult NS_NewMyportalHandlerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGMyportalProtocolHandlerFactory *result = new nsGMyportalProtocolHandlerFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 
 static gint 
@@ -562,6 +532,8 @@ render_search_form (OutStream *stream, BookmarkItem *b)
 
 	for (i = 0; query[i]; ++i) {
 		gchar **param, *smtpos;
+		if (query[i][0] == '\0')
+			continue;
 		if (strcmp (query[i], "%s") == 0)
 		{
 			/* anonymous field */
@@ -574,7 +546,7 @@ render_search_form (OutStream *stream, BookmarkItem *b)
 		if (param[0] && param[1] && (smtpos = strstr (param[1], "%s")))
 		{
 			/* fill in the field with the text minus the %s */
-			gchar *val=g_strdup_printf ("%.*s%s", smtpos-param[1],
+			gchar *val=g_strdup_printf ("%.*s%s", (int)(smtpos-param[1]),
 					param[1],param[1]+(smtpos-param[1]+2));
 			/* add a form input */
 			RENDER (num_inputs>1?param[0]:"");
