@@ -21,6 +21,7 @@
 #include "ProgressListener2.h"
 
 #include "mozilla.h"
+#include "mime.h"
 #include "misc_general.h"
 #include "misc_gui.h"
 #include "glade.h"
@@ -29,7 +30,9 @@
 #include "eel-gconf-extensions.h"
 #include "embed.h"
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <gtk/gtkclist.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkprogress.h>
@@ -42,12 +45,15 @@
 #include <libgnomeui/gnome-popup-menu.h>
 #include <libgnomeui/gnome-app.h>
 #include <libgnomeui/gnome-app-helper.h>
+#include <libgnomeui/gnome-winhints.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "nsXPIDLString.h"
 #include "nsIChannel.h"
+#include "nsIMIMEService.h"
 #include "nsIFTPChannel.h"
-#include "nsIFactory.h"
-#include "nsXPComFactory.h"
+#include "nsCRT.h"
+#include "nsCOMPtr.h"
 
 extern "C" {
 /*
@@ -140,8 +146,8 @@ static void selection_changed (DownloadDialogWidgets *widgets);
 
 static gboolean timeout_listener (GProgressListener2 *Progress);
 
-NS_IMPL_ISUPPORTS2 (GProgressListener2, nsIProgressDialog,
-		    nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS4 (GProgressListener2, nsIDownload, nsIWebProgressListener,
+		    nsIProgressDialog, nsISupportsWeakReference)
 
 //---------------------------------------------------------------------------
 
@@ -150,7 +156,11 @@ NS_IMPL_ISUPPORTS2 (GProgressListener2, nsIProgressDialog,
 GProgressListener2::GProgressListener2 (void) : mLauncher(nsnull),
 						mPersist(nsnull),
 						mHandler(nsnull),
-						mObserver(nsnull)
+						mObserver(nsnull),
+#if MOZILLA_SNAPSHOT > 1
+						mMIMEInfo(nsnull),
+#endif
+						mPercentComplete(0)
 {
 	NS_INIT_ISUPPORTS ();
 }
@@ -166,7 +176,7 @@ NS_METHOD GProgressListener2::InitForPersist (nsIWebBrowserPersist *aPersist,
 					      nsIFile *aFile, 
 					      DownloadAction aAction,
 					      gpointer info,
-					      PRInt64 aTimeDownloadStarted = 0)
+					      PRInt64 aTimeDownloadStarted)
 {
 	nsresult rv;
 
@@ -174,7 +184,7 @@ NS_METHOD GProgressListener2::InitForPersist (nsIWebBrowserPersist *aPersist,
 	mParent = aParent;
 	mAction = aAction;
 	mNoDialog = (mAction == ACTION_FAVICON || mAction == ACTION_SET_PIXMAP ||
-		     mAction == ACTION_ADDBOOKMARK);
+		     mAction == ACTION_ADDBOOKMARK || mAction == ACTION_SILENT);
 	mUri = aURI;
 	mFile = aFile;
 	mPersist = aPersist;
@@ -182,7 +192,7 @@ NS_METHOD GProgressListener2::InitForPersist (nsIWebBrowserPersist *aPersist,
 	mCallbackInfo = info;
 
 	/* do remaining init */
-	rv = Init ();
+	rv = PrivateInit ();
 
 	/* pick up progress messages */
 	mPersist->SetProgressListener (this);
@@ -204,15 +214,22 @@ NS_METHOD GProgressListener2::InitForDownload (nsIHelperAppLauncher *aLauncher,
 	mParent = do_QueryInterface (aContext);
 	mAction = aAction;
 	mNoDialog = (mAction == ACTION_FAVICON || mAction == ACTION_SET_PIXMAP ||
-		     mAction == ACTION_ADDBOOKMARK);
+		     mAction == ACTION_ADDBOOKMARK || mAction == ACTION_SILENT);
 	mHandler = aHandler;
 	mLauncher = aLauncher;
+
+#if MOZILLA_SNAPSHOT > 11
+	rv = mLauncher->GetSource(getter_AddRefs(mUri));
+	rv = mLauncher->GetTimeDownloadStarted(&mTimeDownloadStarted);
+	rv = mLauncher->GetTargetFile(getter_AddRefs(mFile));
+#else
 	rv = mLauncher->GetDownloadInfo (getter_AddRefs (mUri),
 					 &mTimeDownloadStarted,
 					 getter_AddRefs (mFile));
+#endif
 
 	/* do remaining init */
-	rv = Init ();
+	rv = PrivateInit ();
 
 	/* pick up progress messages */
 	mLauncher->SetWebProgressListener (this);
@@ -221,14 +238,12 @@ NS_METHOD GProgressListener2::InitForDownload (nsIHelperAppLauncher *aLauncher,
 	return rv;
 }
 
-NS_METHOD GProgressListener2::Init (void)
+NS_METHOD GProgressListener2::PrivateInit (void)
 {
 	GtkWidget *aParentWidget;
 	GtkCList *clist;
 	nsresult rv;
 
-	g_return_val_if_fail (GTK_IS_CLIST (dialog->clist), NS_ERROR_FAILURE);
-
 	if (mAction == ACTION_FAVICON || mAction == ACTION_FAVICON_EDITOR ||
 	    mAction == ACTION_SET_PIXMAP)
 	{
@@ -258,6 +273,11 @@ NS_METHOD GProgressListener2::Init (void)
 
 	if (!mNoDialog)
 	{
+		if(!GTK_IS_CLIST(dialog->clist))
+		{
+			downloader_dialog_init();
+		}
+
 		/* increment window count while progress is under way */
 		window_count++;
 
@@ -276,10 +296,15 @@ NS_METHOD GProgressListener2::Init (void)
 		}
 
 		/* setup filename */
-		rv = mFile->GetLeafName (&(text[COL_FILENAME]));
-		rv = mFile->GetPath (&(text[COL_DESTINATION]));
+		nsCAutoString leafName;
+		rv = mFile->GetNativeLeafName (leafName);
+		text[COL_FILENAME] = nsCRT::strdup (leafName.get());
 
-		nsCString spec;
+		nsCAutoString path;
+		rv = mFile->GetNativePath (path);
+		text[COL_DESTINATION] = nsCRT::strdup (path.get());
+
+		nsCAutoString spec;
 		rv = mUri ->GetSpec (spec);
 		text[COL_SOURCE] = nsCRT::strdup (spec.get());
 
@@ -302,8 +327,17 @@ NS_METHOD GProgressListener2::Init (void)
 		nsMemory::Free (text[COL_SOURCE]);
 		nsMemory::Free (text[COL_DESTINATION]);
 	
-		/* make sure the dialog is shown */
-		gtk_widget_show_now (dialog->window);
+		/* make sure the dialog is shown, moving it to the current
+		 * workspace if it's mapped and it's not there already */
+		if (GTK_WIDGET_MAPPED (dialog->window) &&
+		    gnome_win_hints_get_current_workspace () !=
+		    gnome_win_hints_get_workspace (dialog->window))
+		{
+			gnome_win_hints_set_workspace (dialog->window,
+				gnome_win_hints_get_current_workspace ());
+		}
+		else gtk_widget_show_now (dialog->window);
+
 		g_assert (clist->rows > 0);
 	}
 
@@ -312,16 +346,49 @@ NS_METHOD GProgressListener2::Init (void)
 }
 
 /* nsIProgressDialog impl */
-
-NS_IMETHODIMP GProgressListener2::Open(nsIDOMWindow *aParent,
-				      nsIWebBrowserPersist *aPersist)
+NS_IMETHODIMP GProgressListener2::Init(nsIURI *aSource,
+				       nsILocalFile *aTarget,
+				       const PRUnichar *aDisplayName,
+#if MOZILLA_SNAPSHOT > 1
+				       nsIMIMEInfo *aMIMEInfo,
+#else
+				       const PRUnichar *aOpeningWith,
+#endif
+				       PRInt64 aStartTime,
+				       nsIWebBrowserPersist *aPersist)
 {
-	mParent = aParent;
+	mUri = aSource;
+	mFile = aTarget;
+	mTimeDownloadStarted = aStartTime;
+	mStartTime = aStartTime;
 	mPersist = aPersist;
+#if MOZILLA_SNAPSHOT > 1
+	mMIMEInfo = aMIMEInfo;
+#endif
+
 	mAction = ACTION_NONE;
+#if MOZILLA_SNAPSHOT > 1
+	if(mMIMEInfo)
+	{
+		nsMIMEInfoHandleAction mimeAction;
+		if(NS_SUCCEEDED(mMIMEInfo->GetPreferredAction(&mimeAction)))
+		{
+			mAction = (mimeAction == nsIMIMEInfo::useHelperApp) ?
+				  ACTION_SAVEFORHELPER : ACTION_NONE;
+		}
+	}
+#endif
 	mNoDialog = 0;
 
-	return Init();
+	return PrivateInit();
+}
+
+NS_IMETHODIMP GProgressListener2::Open(nsIDOMWindow *aParent)
+{
+	mParent = aParent;
+	mNoDialog = 0;
+
+	return NS_OK;
 }
 
 /* attribute long long startTime; */
@@ -330,12 +397,6 @@ NS_IMETHODIMP GProgressListener2::GetStartTime(PRInt64 *aStartTime)
 	*aStartTime = mStartTime;
 	return NS_OK;
 }
-NS_IMETHODIMP GProgressListener2::SetStartTime(PRInt64 aStartTime)
-{
-	mTimeDownloadStarted = aStartTime;
-	mStartTime = aStartTime;
-	return NS_OK;
-}
 
 /* attribute nsIURI source; */
 NS_IMETHODIMP GProgressListener2::GetSource(nsIURI * *aSource)
@@ -343,12 +404,7 @@ NS_IMETHODIMP GProgressListener2::GetSource(nsIURI * *aSource)
 	NS_IF_ADDREF(*aSource = mUri);
 	return NS_OK;
 }
-NS_IMETHODIMP GProgressListener2::SetSource(nsIURI * aSource)
-{
-	mUri = aSource;
-	return NS_OK;
-}
-  
+
 /* attribute nsILocalFile target; */
 NS_IMETHODIMP GProgressListener2::GetTarget(nsILocalFile * *aTarget)
 {
@@ -356,21 +412,20 @@ NS_IMETHODIMP GProgressListener2::GetTarget(nsILocalFile * *aTarget)
 	NS_IF_ADDREF(*aTarget = aLocalFile);
 	return NS_OK;
 }
-NS_IMETHODIMP GProgressListener2::SetTarget(nsILocalFile * aTarget)
+
+#if MOZILLA_SNAPSHOT > 1
+NS_IMETHODIMP GProgressListener2::GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo)
 {
-	mFile = aTarget;
+	NS_IF_ADDREF(*aMIMEInfo = mMIMEInfo);
 	return NS_OK;
 }
-
+#else
 NS_IMETHODIMP GProgressListener2::GetOpeningWith(PRUnichar * *aOpeningWith)
 {
 	*aOpeningWith = NULL;
 	return NS_OK;
 }
-NS_IMETHODIMP GProgressListener2::SetOpeningWith(const PRUnichar * aOpeningWith)
-{
-	return NS_OK;
-}
+#endif
 
 /* attribute nsIObserver observer; */
 NS_IMETHODIMP GProgressListener2::GetObserver(nsIObserver * *aObserver)
@@ -384,10 +439,10 @@ NS_IMETHODIMP GProgressListener2::SetObserver(nsIObserver * aObserver)
 	return NS_OK;
 }
 
-/* readonly attribute nsIWebBrowserPersist operation; */
-NS_IMETHODIMP GProgressListener2::GetOperation(nsIWebBrowserPersist * *aOperation)
+/* readonly attribute nsIWebBrowserPersist persist; */
+NS_IMETHODIMP GProgressListener2::GetPersist(nsIWebBrowserPersist * *aPersist)
 {
-	NS_IF_ADDREF(*aOperation = mPersist);
+	NS_IF_ADDREF(*aPersist = mPersist);
 	return NS_OK;
 }
 
@@ -398,6 +453,53 @@ NS_IMETHODIMP GProgressListener2::GetDialog(nsIDOMWindow * *aDialog)
 	return NS_OK;
 }
 
+#if MOZILLA_SNAPSHOT > 1
+/* attribute PRBool cancelDownloadOnClose; */
+NS_IMETHODIMP  GProgressListener2::GetCancelDownloadOnClose(PRBool *aCancelDownloadOnClose)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+NS_IMETHODIMP  GProgressListener2::SetCancelDownloadOnClose(PRBool aCancelDownloadOnClose)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
+/* nsIDownload Impl */
+
+NS_IMETHODIMP GProgressListener2::SetDialog(nsIDOMWindow *aDialog)
+{
+	return NS_OK;
+}
+
+/* readonly attribute PRInt32 percentComplete; */
+NS_IMETHODIMP GProgressListener2::GetPercentComplete(PRInt32 *aPercentComplete)
+{
+	return *aPercentComplete = mPercentComplete;
+}
+
+/* attribute wstring displayName; */
+NS_IMETHODIMP GProgressListener2::GetDisplayName(PRUnichar * *aDisplayName)
+{
+	*aDisplayName = nsnull;
+	return NS_OK;
+}
+NS_IMETHODIMP GProgressListener2::SetDisplayName(const PRUnichar * aDisplayName)
+{
+	return NS_OK;
+}
+
+/* attribute nsIWebProgressListener listener; */
+NS_IMETHODIMP GProgressListener2::GetListener(nsIWebProgressListener * *aListener)
+{
+	*aListener = nsnull;
+	return NS_OK;
+}
+NS_IMETHODIMP GProgressListener2::SetListener(nsIWebProgressListener * aListener)
+{
+	return NS_OK;
+}
+
 /* nsIWebProgressListener impl */
 
 /*
@@ -408,15 +510,15 @@ NS_IMETHODIMP GProgressListener2::GetDialog(nsIDOMWindow * *aDialog)
  */
 NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 						 nsIRequest *aRequest,
-						 PRInt32 aStateFlags,
+						 PRUint32 aStateFlags,
 						 PRUint32 aStatus)
 {
 	if (mAbort) return NS_ERROR_FAILURE;
 
 	if (aStateFlags & nsIWebProgressListener::STATE_STOP)
 	{
-		nsXPIDLCString filename;
-		mFile->GetPath (getter_Copies (filename));
+		nsCAutoString filename;
+		mFile->GetNativePath (filename);
 
 		switch (mAction)
 		{
@@ -430,7 +532,7 @@ NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 			/* build command */
 			command = g_strconcat ("background-properties-capplet "
 					       "--init-session-settings "
-					       "--ignore --background-image=",
+					       "--background-image=",
 					       filename.get(), NULL);
 
 			/* execute it synchronously */
@@ -441,6 +543,9 @@ NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 			break;
 
 		case ACTION_SAVEFORHELPER:
+#if MOZILLA_SNAPSHOT > 1
+			LaunchHelperApp();
+#else
 			if (mLauncher)
 			{
 				mLauncher->CloseProgressWindow ();
@@ -449,6 +554,7 @@ NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 			{
 				mHandler->LaunchHelperApp ();
 			}
+#endif
 			break;
 
 		case ACTION_ADDBOOKMARK:
@@ -478,7 +584,9 @@ NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 					(SetPixmapInfo *) mCallbackInfo;
 
 				gchar *fn;
-				mFile->GetPath(&fn);
+				nsCAutoString cfn;
+				mFile->GetNativePath(cfn);
+				fn = nsCRT::strdup(cfn.get());
 
 				if (i->pixmap == NULL ||
 				    !GTK_IS_PIXMAP (i->pixmap) ||
@@ -487,7 +595,7 @@ NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 				    strcmp (i->url, i->selected_url))
 				{
 					unlink (fn);
-					g_free (fn);
+					nsMemory::Free (fn);
 					g_free (i);
 					break;
 				}
@@ -521,6 +629,7 @@ NS_IMETHODIMP GProgressListener2::OnStateChange (nsIWebProgress *aWebProgress,
 			break;
 			
 		case ACTION_NONE:
+		case ACTION_SILENT:
 			if (mLauncher)
 			{
 				mLauncher->CloseProgressWindow ();
@@ -575,6 +684,7 @@ NS_IMETHODIMP GProgressListener2::
 	int row = gtk_clist_find_row_from_data (clist, this);
 	DetailsInfo *dinfo = (DetailsInfo *)
 		g_hash_table_lookup (details_hash, this);
+	if (!dinfo) return NS_OK;
 	char buffer[256];
 
 	if (!mCheckedCanPause)
@@ -643,6 +753,14 @@ NS_IMETHODIMP GProgressListener2::
 	{
 		gfloat progress = (gfloat)aCurTotalProgress /
 				  (gfloat)aMaxTotalProgress;
+		/* handle the case where mozilla gunzips on the fly and
+		 * aCurTotalProgress becomes bigger than aMaxTotalProgress */
+		if (progress > 1)
+		{
+			progress = 1;
+		}
+
+		mPercentComplete = NS_STATIC_CAST(PRInt32, progress);
 
 		sprintf (buffer, "%.1f%%", progress * 100.0);
 		gtk_clist_set_text (clist, row, COL_PERCENT, buffer);
@@ -758,7 +876,8 @@ NS_IMETHODIMP GProgressListener2::
 /* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
 NS_IMETHODIMP GProgressListener2::
 			OnSecurityChange(nsIWebProgress *aWebProgress,
-					 nsIRequest *aRequest, PRInt32 state)
+					 nsIRequest *aRequest,
+					 PRUint32 state)
 {
     return NS_OK;
 }
@@ -785,6 +904,66 @@ void GProgressListener2::FormatTime (PRUint32 aTime, char *buffer)
 
 //---------------------------------------------------------------------------
 
+NS_METHOD GProgressListener2::LaunchHelperApp (void)
+{
+	if (!mMIMEInfo)
+		return NS_ERROR_FAILURE;
+
+	nsresult rv;
+
+	nsCOMPtr<nsIFile> helperFile;
+	rv = mMIMEInfo->GetPreferredApplicationHandler(getter_AddRefs(helperFile));
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	nsCAutoString helperFileName;
+	rv = helperFile->GetNativePath(helperFileName);
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	nsMIMEInfoHandleAction mimeAction;
+	rv = mMIMEInfo->GetPreferredAction(&mimeAction);
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIExternalHelperAppService> helperService =
+		do_GetService (NS_EXTERNALHELPERAPPSERVICE_CONTRACTID, &rv);
+	if (NS_SUCCEEDED(rv))
+	{
+		nsCOMPtr<nsPIExternalAppLauncher> appLauncher =
+			do_QueryInterface (helperService, &rv);
+		if (NS_SUCCEEDED(rv))
+		{
+			appLauncher->DeleteTemporaryFileOnExit(mFile);
+		}
+	}
+
+	nsCAutoString aFileName;
+	mFile->GetNativePath(aFileName);
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	gchar *tmp_dir = g_get_tmp_dir ();
+
+	nsXPIDLString helperDesc;
+	rv = mMIMEInfo->GetApplicationDescription(getter_Copies(helperDesc));
+	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+	if(helperDesc.Equals(NS_LITERAL_STRING("runInTerminal")))
+	{
+		const char *argv[4] = { "gnome-terminal", "-e",
+					helperFileName.get(),
+					aFileName.get() };
+		gnome_execute_async (tmp_dir, 4, const_cast<char **>(argv));
+	}
+	else
+	{
+		const char *argv[2] = { helperFileName.get(),
+					aFileName.get() };
+		gnome_execute_async (tmp_dir, 2, const_cast<char **>(argv));
+	}
+
+	return NS_OK;
+}
+
+//---------------------------------------------------------------------------
+
 nsresult GProgressListener2::Cleanup (void)
 {
 	gint row;
@@ -927,28 +1106,6 @@ PRBool GProgressListener2::CanResume (void)
 	return (mCanPause && mIsPaused);
 }
 
-NS_DEF_FACTORY (GProgressListener2, GProgressListener2);
-
-/**
- * NS_NewProgressListenerFactory:
- */ 
-nsresult NS_NewProgressListenerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGProgressListener2Factory *result = new nsGProgressListener2Factory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
-
 /*
  * Global dialog initialisation
  */
@@ -1161,6 +1318,14 @@ download_clist_button_press_event_cb (GtkCList *clist,
 	
 		/* show context menu */
 		menu = gnome_popup_menu_new (menu_uiinfo);
+		
+		/*accout for the case where nothing is selected */
+		if(!dinfo)
+		{
+			gtk_widget_set_sensitive (menu_uiinfo[0].widget, FALSE);
+			gtk_widget_set_sensitive (menu_uiinfo[1].widget, FALSE);
+		}
+		
 		action = gnome_popup_menu_do_popup_modal 
 			(GTK_WIDGET (menu), NULL, NULL, event, NULL);
 
