@@ -24,12 +24,8 @@
 #include "themes.h"
 
 #include "nsCOMPtr.h"
-#include "nsIFactory.h"
 #include "nsString.h"
 #include "nsIServiceManager.h"
-#include "nsXPComFactory.h"
-
-#include "nsIPromptService.h"
 #include "nsIUnicodeEncoder.h"
 
 #include <gtk/gtkentry.h>
@@ -40,6 +36,8 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-triggers.h>
 
+#include "PromptService.h"
+
 /* local function prototypes */
 static void set_title (GtkWidget *dialog, const PRUnichar *title);
 static void set_label_text (GtkWidget *label, const PRUnichar *text);
@@ -52,21 +50,6 @@ static void set_editable (GtkWidget *entry, PRUnichar **text);
 static void get_check_button (GtkWidget *button, PRBool *value);
 static void get_editable (GtkWidget *editable, PRUnichar **text);
 
-/**
- * class CPromptService: an GNOME implementation of prompt dialogs for
- * Mozilla
- */
-class CPromptService: public nsIPromptService
-{
-	public:
-		CPromptService();
-  		virtual ~CPromptService();
-
-		NS_DECL_ISUPPORTS
-		NS_DECL_NSIPROMPTSERVICE
-  
-};
-
 NS_IMPL_ISUPPORTS1 (CPromptService, nsIPromptService)
 
 /**
@@ -74,7 +57,7 @@ NS_IMPL_ISUPPORTS1 (CPromptService, nsIPromptService)
  */
 CPromptService::CPromptService ()
 {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 }
 
 /**
@@ -626,9 +609,14 @@ NS_IMETHODIMP CPromptService::Select (nsIDOMWindow *parent,
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
 	/* handle return values */
-	GtkWidget *selectedItem = GTK_WIDGET(GTK_LIST(list)->selection->data);
-	*outSelection = gtk_list_child_position(GTK_LIST(list), selectedItem);
-	*_retval = (ret == 0);
+	if(g_list_length(GTK_LIST(list)->selection))
+	{
+		GtkWidget *selectedItem = GTK_WIDGET(GTK_LIST(list)->selection->data);
+		*outSelection = gtk_list_child_position(GTK_LIST(list), selectedItem);
+		*_retval = (ret == 0) ? PR_TRUE : PR_FALSE;
+	}
+	else
+		*_retval = PR_FALSE;
 
 	/* done */
 	gtk_list_clear_items (GTK_LIST(list), 0 ,-1);
@@ -636,28 +624,6 @@ NS_IMETHODIMP CPromptService::Select (nsIDOMWindow *parent,
 	return NS_OK;
 }
 
-NS_DEF_FACTORY (CPromptService, CPromptService);
-
-/**
- * NS_NewPromptServiceFactory:
- */ 
-nsresult NS_NewPromptServiceFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsCPromptServiceFactory *result = new nsCPromptServiceFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-  
-	return NS_OK;
-}
-
 /**
  * set_title: set a dialog title to a unicode string
  */
