@@ -18,7 +18,6 @@
 
 #include "galeon.h"
 #include "prefs.h"
-#include "outputbuffer.h"
 
 #include <algorithm>
 
@@ -76,9 +75,10 @@ using namespace std;
 #define RENDER_STL_STR(str) \
 	oStream->Write (str.c_str(), str.size(), &bytesWriten);
 
-void ParseEnvPath(const nsCString &path, list<string> &dirs);
-int gHelpSelect (const struct dirent *dirEntry);
-int gnomeHelpSelect (const struct dirent *dirEntry);
+static void ParseEnvPath(const nsACString &path, list<string> &dirs);
+static int gHelpSelect (const struct dirent *dirEntry);
+static int gnomeHelpSelect (const struct dirent *dirEntry);
+static void RenderContentType (nsIOutputStream *oStream, PRUint32 &bytesWriten);
 
 /* Implementation file */
 NS_IMPL_ISUPPORTS1 (GTOCProtocolHandler, nsIProtocolHandler)
@@ -107,29 +107,36 @@ NS_METHOD GTOCProtocolHandler::CreatePage (void)
 	if (NS_FAILED(rv)) return rv;
 
 	/* render appropriate content */
-	if (!nsCRT::strcasecmp("info", mDocType.get()))
+	if (mDocType.Equals("info"))
 		rv = CreateInfoPage ();
-	else if (!nsCRT::strcasecmp("man", mDocType.get()))
+	else if (mDocType.Equals("man"))
 		rv = CreateManPage();
-	else if (!nsCRT::strcasecmp("ghelp", mDocType.get()))
+	else if (mDocType.Equals("ghelp"))
 		rv = CreateHelpPage ("ghelp", gHelpSelect);
-	else if (!nsCRT::strcasecmp("gnome-help", mDocType.get()))
+	else if (mDocType.Equals("gnome-help"))
 		rv = CreateHelpPage ("gnome-help", gnomeHelpSelect);
 	else
 		rv = CreateTOCPage();
 	if (NS_FAILED(rv)) return rv;
 
 	/* finish the rendering */
-	PRUint32 size;  
-	rv = mStream->GetLength(&size);
-	if (NS_FAILED(rv)) return rv;
-
 	nsCOMPtr<nsIInputStream> iStream;
 	rv = mStream->NewInputStream(0, getter_AddRefs(iStream));
 	if (NS_FAILED(rv)) return rv;
 
+#if MOZILLA_SNAPSHOT > 4
 	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
-				      iStream, "text/html", size);
+				      iStream, NS_LITERAL_CSTRING("text/html"),
+				      NS_LITERAL_CSTRING("utf-8"));
+#else
+	PRUint32 size;
+	rv = mStream->GetLength(&size);
+	if (NS_FAILED(rv)) return rv;
+
+	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
+				      iStream, NS_LITERAL_CSTRING("text/html"),
+				      NS_LITERAL_CSTRING("utf-8"), size);
+#endif
 	if (NS_FAILED(rv)) return rv;
 
 	return rv;
@@ -153,21 +160,7 @@ NS_METHOD GTOCProtocolHandler::CreateTOCPage (void)
 	RENDER (_("GNOME Help Index"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-	nsCString charset;
-	charset.AssignWithConversion (platformCharset);
-
-	if (!charset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (charset.get());
-		RENDER ("\">");
-	}
+	RenderContentType(oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 
@@ -202,13 +195,13 @@ NS_METHOD GTOCProtocolHandler::CreateInfoPage (void)
 {
 	nsresult rv;
 
-	char *cinfopath = g_strconcat ("/usr/info:/usr/local/info:",
-				       g_getenv("INFOPATH"), NULL);
-	char *cinfopath_aux = g_strconcat ("INFOPATH=", cinfopath, NULL);
-	putenv (cinfopath_aux);
-	g_free (cinfopath_aux);
-	nsCAutoString infopath(cinfopath);
-	g_free (cinfopath);
+	const nsCAutoString infopath(NS_LITERAL_CSTRING("/usr/info:/usr/local/info:") +
+				     nsDependentCString(g_getenv("INFOPATH")));
+
+	const nsCAutoString infoEnv(NS_LITERAL_CSTRING("INFOPATH=") + infopath);
+	char *auxEnv = strdup(infoEnv.get());
+	putenv (auxEnv);
+
 	list<string> dirs;
 	ParseEnvPath (infopath, dirs);
 
@@ -240,21 +233,7 @@ NS_METHOD GTOCProtocolHandler::CreateInfoPage (void)
 	RENDER (_("pages"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-	nsCString charset;
-	charset.AssignWithConversion (platformCharset);
-
-	if (!charset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (charset.get());
-		RENDER ("\">");
-	}
+	RenderContentType(oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 	RENDER ("<h3><strong>Info pages</strong></h3>\n");
@@ -290,18 +269,19 @@ NS_METHOD GTOCProtocolHandler::CreateManPage (void)
 {
 	nsresult rv;
 
-	nsCAutoString manpath = g_getenv ("MANPATH");
 	list<string> dirs;
-	ParseEnvPath (manpath, dirs);
+	ParseEnvPath (nsDependentCString(g_getenv("MANPATH")), dirs);
 
 	if (dirs.size()>0)
 	{
-		dirs.push_front("/usr/local/man");
-		dirs.push_front("/usr/man");
 		dirs.sort ();
 		dirs.unique ();
-		dirs.erase(find(dirs.begin(), dirs.end(), "/usr/man"));
-		dirs.erase(find(dirs.begin(), dirs.end(), "/usr/local/man"));
+
+		list<string>::iterator i;
+		i = find(dirs.begin(), dirs.end(), "/usr/man");
+		if(i != dirs.end()) dirs.erase(i);
+		i = find(dirs.begin(), dirs.end(), "/usr/local/man");
+		if(i != dirs.end()) dirs.erase(i);
 	}
 	dirs.push_front("/usr/local/man");
 	dirs.push_front("/usr/man");
@@ -324,21 +304,7 @@ NS_METHOD GTOCProtocolHandler::CreateManPage (void)
 	RENDER (_("pages"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-	nsCString charset;
-	charset.AssignWithConversion (platformCharset);
-
-	if (!charset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (charset.get());
-		RENDER ("\">");
-	}
+	RenderContentType(oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 	RENDER ("<h3><strong>Man pages</strong></h3>");
@@ -423,21 +389,7 @@ NS_METHOD GTOCProtocolHandler::CreateHelpPage (const char *type,
 	RENDER (_("pages"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-	nsCString charset;
-	charset.AssignWithConversion (platformCharset);
-
-	if (!charset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (charset.get());
-		RENDER ("\">");
-	}
+	RenderContentType(oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 	RENDER ("<h3><strong>Help pages</strong></h3>");
@@ -460,8 +412,9 @@ NS_METHOD GTOCProtocolHandler::CreateHelpPage (const char *type,
 	return NS_OK;
 }
 
-void ParseEnvPath(const nsCString &path, list<string> &dirs)
+void ParseEnvPath(const nsACString &aPath, list<string> &dirs)
 {
+	const nsCString path(aPath);
 	if (path.Length() != 0)
 	{
 		PRInt32 n = 0, m = path.Find (":"), t;
@@ -479,9 +432,12 @@ void ParseEnvPath(const nsCString &path, list<string> &dirs)
 				n = m;
 				m = path.Find(":",PR_FALSE,n+1);
 			}
-			PRInt32 end = item.Length() - n -1;
-			path.Mid(item, n+1, end);
-			dirs.push_back(item.get());
+			PRInt32 end = path.Length();
+			if(n+1 < end)
+			{
+				path.Mid(item, n+1, end);
+				dirs.push_back(item.get());
+			}
 		}
 		else
 		{
@@ -504,8 +460,8 @@ int gHelpSelect (const struct dirent *dirEntry)
 
 int gnomeHelpSelect (const struct dirent *dirEntry)
 {
-	nsCString fileName = dirEntry->d_name;
-	fileName += ".sgml";
+	nsCAutoString fileName(nsDependentCString(dirEntry->d_name) +
+			       NS_LITERAL_CSTRING(".sgml"));
 
 	char *helpPath = gnome_help_file_find_file (
 					const_cast<char *>(dirEntry->d_name),
@@ -517,6 +473,31 @@ int gnomeHelpSelect (const struct dirent *dirEntry)
 	return ret;
 }
 
+void RenderContentType (nsIOutputStream *oStream, PRUint32 &bytesWriten)
+{
+	nsresult rv;
+	nsCOMPtr<nsIPlatformCharset> pc =
+		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
+	g_return_if_fail (NS_SUCCEEDED(rv));
+	
+#if MOZILLA_SNAPSHOT > 9
+	nsCAutoString platformCharset;
+#else
+	nsAutoString platformCharset;
+#endif
+	rv = pc->GetCharset(kPlatformCharsetSel_Menu, platformCharset);
+	if (!platformCharset.IsEmpty())
+	{
+		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
+#if MOZILLA_SNAPSHOT > 9
+		RENDER (platformCharset.get());
+#else
+		RENDER (NS_ConvertUCS2toUTF8(platformCharset).get());
+#endif
+		RENDER ("\">");
+	}
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 
