@@ -22,6 +22,7 @@
 #include "GaleonWrapper.h"
 #include "EventContext.h"
 #include "misc_string.h"
+#include "SideBar.h"
 
 #include <time.h>
 #include <gdk/gdkkeysyms.h>
@@ -39,7 +40,11 @@
 #include "nsXPIDLString.h"
 #include "nsIPrefService.h"
 #include "nsICharsetConverterManager.h"
+#if MOZILLA_SNAPSHOT > 9
+#include "nsIStringEnumerator.h"
+#else
 #include "nsICharsetConverterManager2.h"
+#endif
 #include "nsIUnicodeEncoder.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsICacheService.h"
@@ -80,6 +85,10 @@
 #include "JSConsoleListener.h"
 #include "nsIDOMCharacterData.h"
 #include "nsIURL.h"
+#if MOZILLA_SNAPSHOT <= 12
+#include "nsCCookieManager.h"
+#endif
+#include "nsCPasswordManager.h"
 
 #define UTF8_ENCODER_ID NS_UNICODEENCODER_CONTRACTID_BASE"UTF-8"  
 #define UTF8_DECODER_ID NS_UNICODEDECODER_CONTRACTID_BASE"UTF-8"
@@ -88,6 +97,11 @@ static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
 static NS_DEFINE_CID(kJVMManagerCID, NS_JVMMANAGER_CID);
 static NS_DEFINE_CID(kPlatformCharsetCID, NS_PLATFORMCHARSET_CID);
 
+#if MOZILLA_SNAPSHOT > 9
+//Little mapping table for permission api. bleah
+const char* gPermType[2] = { "cookie", "image" };
+#endif
+
 #define GALEON_WRAPPER(wrapper)            \
 GaleonWrapper *wrapper;                    \
 return_if_not_embed(embed);	           \
@@ -100,8 +114,14 @@ return_val_if_not_embed (embed, fail);	             \
 g_return_val_if_fail (embed->wrapper != NULL, fail); \
 wrapper = (GaleonWrapper *)embed->wrapper;
 
+#define GALEON_NAMED_WRAPPER_IF_FAIL(embed, gWrapper, fail)	\
+GaleonWrapper *gWrapper;						\
+return_val_if_not_embed (embed, fail);				\
+g_return_val_if_fail (embed->wrapper != NULL, fail);		\
+gWrapper = (GaleonWrapper *)embed->wrapper;
+
 /* local function prototypes */
-static char  *convert_ns_string_to_c_string (const nsString & ns_string);
+static char  *convert_ns_string_to_c_string (const nsAString & ns_string);
 static gboolean stylesheet_is_alternate (nsIDOMStyleSheet *item);
 
 extern "C" gboolean
@@ -122,7 +142,12 @@ extern "C" gboolean
 mozilla_preference_set(const char *preference_name, const char *new_value)
 {
 	g_return_val_if_fail (preference_name != NULL, FALSE);
-	g_return_val_if_fail (new_value != NULL, FALSE);
+
+	/*It is legitimate to pass in a NULL value sometimes. So let's not
+	 *assert and just check and return.
+	 */
+	if (!new_value) return FALSE;
+
 	nsCOMPtr<nsIPrefService> prefService = 
 				do_GetService (NS_PREFSERVICE_CONTRACTID);
 	nsCOMPtr<nsIPrefBranch> pref;
@@ -317,14 +342,17 @@ mozilla_wrapper_destroy (GaleonEmbed *embed)
 }
 
 extern "C" gboolean mozilla_find (GaleonEmbed *embed, 
-				  WrapperSearchProperties *properties)
+				  WrapperSearchProperties *properties,
+				  gboolean interactive)
 {
 	PRUnichar *search_string;		
 	PRBool didFind;	
 	GALEON_WRAPPER_IF_FAIL (wrapper, FALSE)
 
 	search_string = mozilla_locale_to_unicode (properties->search_string);
-	wrapper->Find (search_string, properties->match_case, 
+	wrapper->Find (search_string,
+		       interactive ? PR_TRUE : PR_FALSE,
+		       properties->match_case, 
 		       properties->backwards, properties->wrap,
 		       properties->entire_word, properties->search_frames,
 		       &didFind);
@@ -348,8 +376,8 @@ extern "C" gboolean mozilla_save_url (GaleonEmbed *embed, const char *url,
 	{
 		nsresult rv;
 
-		nsString s;
-		s.AssignWithConversion(url);
+		const nsAutoString &s = NS_ConvertUTF8toUCS2(url);
+
 		nsCOMPtr<nsIURI> linkURI;
 		rv = NS_NewURI(getter_AddRefs(linkURI), s);
 		if (NS_FAILED(rv) || !linkURI) return NS_ERROR_FAILURE;
@@ -359,7 +387,7 @@ extern "C" gboolean mozilla_save_url (GaleonEmbed *embed, const char *url,
 		if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
 
 		nsCOMPtr<nsILocalFile> file;
-		NS_NewLocalFile(filename, PR_TRUE, getter_AddRefs(file)); 
+		NS_NewNativeLocalFile(nsDependentCString(filename), PR_TRUE, getter_AddRefs(file)); 
 		if (NS_FAILED(rv) || !file) return NS_ERROR_FAILURE;
 
 		GProgressListener2 *aProgress = new GProgressListener2 ();
@@ -374,7 +402,11 @@ extern "C" gboolean mozilla_save_url (GaleonEmbed *embed, const char *url,
 		else
 			persist->SetPersistFlags (nsIWebBrowserPersist::PERSIST_FLAGS_NONE);
 
+#if MOZILLA_SNAPSHOT > 4
+		return persist->SaveURI(linkURI, nsnull, nsnull, nsnull, nsnull, file);
+#else
 		return persist->SaveURI(linkURI, nsnull, file);
+#endif
 	}
 
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
@@ -401,6 +433,30 @@ extern "C" gboolean mozilla_reload (GaleonEmbed *embed)
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
+/**
+ * mozilla_copy_page: Copies the contents of a DOMWindow to another.
+ * This is used for creating view-source and print-preview views.
+ **/
+
+extern "C" gboolean
+mozilla_copy_page (GaleonEmbed *dest, GaleonEmbed *source, guint aDisplayType)
+{
+	GALEON_NAMED_WRAPPER_IF_FAIL(dest, dWrapper, FALSE);
+	GALEON_NAMED_WRAPPER_IF_FAIL(source, sWrapper, FALSE);
+
+	nsresult rv;
+
+	nsCOMPtr<nsISupports> aPageDescriptor;
+	rv = sWrapper->GetPageDescriptor(getter_AddRefs(aPageDescriptor));
+	if (!aPageDescriptor || NS_FAILED(rv)) return FALSE;
+
+	rv = dWrapper->LoadDocument(aPageDescriptor, aDisplayType);
+	if (NS_FAILED(rv)) return FALSE;
+
+	return TRUE;
+}
+
+
 /**
  * mozilla_set_zoom: Sets the zoom factor of a embed
  */
@@ -443,6 +499,11 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info, gboolean preview)
 		nsIPrintSettings::kSelectedFrame,
 		nsIPrintSettings::kEachFrameSep
 	};
+	/* these should match the order of the radiobuttons in the dialog 
+	 * and the paper names in the default print provider PS*/
+	const static char *PaperSizeNames[] = {
+		"Letter","Legal","Executive","A4"
+	};
 
 	nsCOMPtr<nsIPrintOptions> printService(do_GetService(kPrintOptionsCID));
 	g_assert(printService);
@@ -454,6 +515,7 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info, gboolean preview)
 	switch (info->pages)
 	{
 	case 0:
+		options->SetPrintRange (nsIPrintSettings::kRangeAllPages);
 		break;
 	case 1:
 		options->SetPrintRange (nsIPrintSettings::kRangeSpecifiedPageRange);
@@ -473,7 +535,7 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info, gboolean preview)
 	PRUnichar * tempstr;
 
 	tempstr = mozilla_locale_to_unicode("PostScript/default");
-	printService->SetPrinterName(tempstr);
+	options->SetPrinterName(tempstr);
 	g_free(tempstr);
 
 	tempstr = mozilla_locale_to_unicode(info->header_left_string);
@@ -510,8 +572,16 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info, gboolean preview)
 	options->SetPrintCommand (tempstr);
 	g_free(tempstr);
 
+	/* native paper size formats. Our dialog does not support custom yet */
+	options->SetPaperSize (nsIPrintSettings::kPaperSizeNativeData);
+	if (info->paper < 0 || info->paper > 3)
+	{
+		info->paper = 0;
+	}
+	options->SetPaperName (
+		NS_ConvertUTF8toUCS2(PaperSizeNames[info->paper]).get());
+	
 	options->SetPrintReversed (info->reversed);
-	options->SetPaperSize (info->paper);
 	options->SetPrintInColor (info->print_color);
 	options->SetOrientation (info->orientation);
 	options->SetPrintFrameType (frame_types[info->frame_type]);
@@ -519,9 +589,51 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info, gboolean preview)
 	options->SetPrintSilent (PR_TRUE);
 
 	result = wrapper->Print(options, preview);
+
+	if(preview) print_show_preview_controls(embed);
+
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
+/**
+ * mozilla_print_preview_close: Revert a print preview DOMWindow
+ * to normal viewing.
+ */
+
+extern "C"
+gboolean mozilla_print_preview_close (GaleonEmbed *embed)
+{
+	GALEON_WRAPPER_IF_FAIL(wrapper, FALSE)
+
+	nsresult rv = wrapper->PrintPreviewClose();
+	return NS_SUCCEEDED(rv) ? TRUE : FALSE;
+}
+
+/**
+ * mozilla_print_preview_num_pages: Get number of pages in print preview
+ */
+
+extern "C"
+gint mozilla_print_preview_num_pages (GaleonEmbed *embed)
+{
+	GALEON_WRAPPER_IF_FAIL(wrapper, 0)
+
+	gint numPages = 0;
+	wrapper->PrintPreviewNumPages(&numPages);
+
+	return numPages;
+}
+
+extern "C"
+gboolean mozilla_print_preview_navigate (GaleonEmbed *embed, short navType,
+					 gint pageNum)
+{
+	GALEON_WRAPPER_IF_FAIL(wrapper, FALSE)
+
+	nsresult rv = wrapper->PrintPreviewNavigate(navType, pageNum);
+	return NS_SUCCEEDED(rv) ? TRUE : FALSE;
+}	
+
 /**
  * mozilla_session_history_get_url: Gets the session history and  ..
  * GtkMozWrapper. On return,  ..
@@ -532,7 +644,7 @@ mozilla_session_history_get_url (GaleonEmbed *embed, int index)
 	GALEON_WRAPPER_IF_FAIL(wrapper, NULL)
 	
 	nsresult rv;
-	nsCString url;
+	nsCAutoString url;
 
 	rv = wrapper->GetSHUrlAtIndex(index, url);
 
@@ -607,7 +719,7 @@ mozilla_get_main_document_url (GaleonEmbed *embed)
 {
 	GALEON_WRAPPER_IF_FAIL(wrapper, NULL)
 	
-	nsCString url;
+	nsCAutoString url;
 	nsresult rv = wrapper->GetMainDocumentUrl (url);
 
 	return (NS_SUCCEEDED (rv) && !url.IsEmpty()) ? g_strdup(url.get()) : NULL;
@@ -622,7 +734,7 @@ mozilla_get_document_url (GaleonEmbed *embed)
 {
 	GALEON_WRAPPER_IF_FAIL(wrapper, NULL)
 	
-	nsCString url;
+	nsCAutoString url;
 	nsresult rv = wrapper->GetDocumentUrl (url);
 
 	return (NS_SUCCEEDED (rv) && !url.IsEmpty()) ? g_strdup(url.get()) : NULL;
@@ -659,7 +771,7 @@ mozilla_get_document_title (GaleonEmbed *embed, gchar **title_utf8)
 	/* do get utf8 version too */
 	if (title_utf8 != NULL)
 	{
-		*title_utf8 = mozilla_unicode_to_utf8 (unicode_title);
+		*title_utf8 = g_strdup(NS_ConvertUCS2toUTF8(unicode_title).get());
 	}
 
 	/* attempt conversion */
@@ -691,7 +803,7 @@ mozilla_get_link_message (GaleonEmbed *embed, gchar **link_message_utf8)
 	if (link_message_utf8 != NULL)
 	{
 		*link_message_utf8
-		    = mozilla_unicode_to_utf8 (unicode_link_message);
+		    = g_strdup(NS_ConvertUCS2toUTF8(unicode_link_message).get());
 	}
 
 	/* attempt conversion */
@@ -723,7 +835,7 @@ mozilla_get_js_status (GaleonEmbed *embed, gchar **js_status_utf8)
 	if (js_status_utf8 != NULL)
 	{
 		*js_status_utf8
-		    = mozilla_unicode_to_utf8 (unicode_js_status);
+		    = g_strdup(NS_ConvertUCS2toUTF8(unicode_js_status).get());
 	}
 
 	/* attempt conversion */
@@ -883,6 +995,8 @@ mozilla_get_mouse_event_info (GaleonEmbed *embed, gpointer event,
 	event_context.Init ((nsIDOMEvent*)event, wrapper);
 	result = event_context.GetMouseEventInfo (context);
 
+	if (NS_FAILED(result)) return FALSE;
+
 	nsCOMPtr<nsIDOMDocument> domDoc;
 	result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
 	if (NS_SUCCEEDED(result))
@@ -940,17 +1054,17 @@ mozilla_free_context_info_sub (WrapperContextInfo *ctx)
  * mozilla_list_cookies: get a list of all saved cookies
  */
 extern "C" GList *
-mozilla_list_cookies (void)
+mozilla_list_cookies(void)
 {
 	GList *cookies = NULL;
 	nsresult result;
 
 	nsCOMPtr<nsICookieManager> cookieManager = 
-			do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+			do_GetService(NS_COOKIEMANAGER_CONTRACTID);
 	nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
 	result = 
-	    cookieManager->GetEnumerator (getter_AddRefs(cookieEnumerator));
-	g_return_val_if_fail (NS_SUCCEEDED(result), NULL);	
+	    cookieManager->GetEnumerator(getter_AddRefs(cookieEnumerator));
+	g_return_val_if_fail(NS_SUCCEEDED(result), NULL);	
 	PRBool enumResult;
 	for (cookieEnumerator->HasMoreElements(&enumResult) ;
 	     enumResult == PR_TRUE ;
@@ -959,36 +1073,39 @@ mozilla_list_cookies (void)
 		Cookie *c;
 	
 		nsCOMPtr<nsICookie> nsCookie;
-		result = cookieEnumerator->GetNext (getter_AddRefs(nsCookie));
-		g_return_val_if_fail (NS_SUCCEEDED(result), NULL);
+		result = cookieEnumerator->GetNext(getter_AddRefs(nsCookie));
+		g_return_val_if_fail(NS_SUCCEEDED(result), NULL);
 
-		c = g_new0 (Cookie, 1);
+		c = g_new0(Cookie, 1);
 
-		nsXPIDLCString transfer;
+		nsCAutoString transfer;
+
+		nsCookie->GetHost(transfer);
+		c->base.domain = g_strdup(transfer.get());
+		nsCookie->GetName(transfer);
+		c->name = g_strdup(transfer.get());
+		nsCookie->GetValue(transfer);
+		c->value = g_strdup(transfer.get());
+		nsCookie->GetPath(transfer);
+		c->path = g_strdup(transfer.get());
 
-		nsCookie->GetHost (getter_Copies(transfer));
-		c->base.domain = g_strdup (transfer.get());
-		nsCookie->GetName (getter_Copies(transfer));
-		c->name = g_strdup (transfer.get());
-		nsCookie->GetValue (getter_Copies(transfer));
-		c->value = g_strdup (transfer.get());
-		nsCookie->GetPath (getter_Copies(transfer));
-		c->path = g_strdup (transfer.get());
-		
 		PRBool isSecure;
-		nsCookie->GetIsSecure (&isSecure);
+		nsCookie->GetIsSecure(&isSecure);
 		if (isSecure == PR_TRUE) 
-			c->secure = g_strdup (_("Yes"));
+			c->secure = g_strdup(_("Yes"));
 		else 
-			c->secure = g_strdup (_("No"));
+			c->secure = g_strdup(_("No"));
 
 		PRUint64 dateTime;
-		nsCookie->GetExpires (&dateTime);
-		c->expire = g_strdup_printf ("%s",ctime((time_t*)&dateTime));
+		nsCookie->GetExpires(&dateTime);
+		if(dateTime == 0)
+			c->expire = g_strdup(_("End of current session"));
+		else 
+			c->expire = g_strdup_printf("%s",ctime((time_t*)&dateTime));
 		
-		cookies = g_list_prepend (cookies, c);
+		cookies = g_list_prepend(cookies, c);
 	}	
-	cookies = g_list_reverse (cookies);
+	cookies = g_list_reverse(cookies);
 	return cookies;
 }
 
@@ -998,16 +1115,16 @@ mozilla_list_cookies (void)
  */
 
 extern "C" GList *
-mozilla_get_permissions (int type)
+mozilla_get_permissions (guint type)
 {
 	GList *permissions = NULL;
 	nsresult result;
 	nsCOMPtr<nsIPermissionManager> permissionManager = 
-			do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
+			do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 	nsCOMPtr<nsISimpleEnumerator> permissionEnumerator;
 	result = permissionManager->
-			GetEnumerator (getter_AddRefs(permissionEnumerator));
-	g_return_val_if_fail (NS_SUCCEEDED(result), NULL);
+			GetEnumerator(getter_AddRefs(permissionEnumerator));
+	g_return_val_if_fail(NS_SUCCEEDED(result), NULL);
 	PRBool enumResult;
 	for (permissionEnumerator->HasMoreElements(&enumResult) ;
 	     enumResult == PR_TRUE ;
@@ -1015,32 +1132,57 @@ mozilla_get_permissions (int type)
 	{
 		nsCOMPtr<nsIPermission> nsPermission;
 		result = permissionEnumerator->
-				GetNext (getter_AddRefs(nsPermission));
-		g_return_val_if_fail (NS_SUCCEEDED(result), NULL);		
+				GetNext(getter_AddRefs(nsPermission));
+		g_return_val_if_fail(NS_SUCCEEDED(result), NULL);		
 
+#if MOZILLA_SNAPSHOT > 9
+		nsCAutoString cType;
+		nsPermission->GetType(cType);
+		if (cType.Equals(gPermType[type]))
+		{
+#else
+	#if MOZILLA_SNAPSHOT > 6
+		PRUint32 cType;
+	#else
 		PRInt32 cType;
-		nsPermission->GetType (&cType);
+	#endif
+		nsPermission->GetType(&cType);
 		if (cType == type)
 		{
-			BlockedHost *b = g_new0 (BlockedHost, 1);
-			gchar *tmp = NULL;
+#endif
+			BlockedHost *b = g_new0(BlockedHost, 1);
 
-			nsPermission->GetHost (&tmp);
-			b->domain = g_strdup (tmp);
-			nsMemory::Free (tmp);
+#if MOZILLA_SNAPSHOT > 6
+			nsCString host;
+			nsPermission->GetHost(host);
+			b->domain = g_strdup(host.get());
+
+			PRUint32 cap;
+			nsPermission->GetCapability(&cap);
+			if (cap == nsIPermissionManager::ALLOW_ACTION) 
+				b->type = g_strdup(_("Allowed"));
+			else if (cap == nsIPermissionManager::DENY_ACTION)
+				b->type = g_strdup(_("Blocked"));
+			else
+				b->type = g_strdup(_("Unknown"));
+#else
+			nsXPIDLCString host;
+			nsPermission->GetHost(getter_Copies(host));
+			b->domain = g_strdup(host.get());
 
 			PRBool cap;
-			nsPermission->GetCapability (&cap);
-			if (cap == PR_TRUE) 
-				b->type = g_strdup (_("Allowed"));
-			else 
-				b->type = g_strdup (_("Blocked"));
+			nsPermission->GetCapability(&cap);
+			if (cap == PR_TRUE)
+				b->type = g_strdup(_("Allowed"));
+			else
+				b->type = g_strdup(_("Blocked"));
+#endif
 
-			permissions = g_list_prepend (permissions, b);
+			permissions = g_list_prepend(permissions, b);
 		}
 	}
 
-	permissions = g_list_reverse (permissions);
+	permissions = g_list_reverse(permissions);
 	return permissions;
 };
 
@@ -1048,12 +1190,30 @@ mozilla_get_permissions (int type)
  * mozilla_set_url_permission: change permissions for a URL
  */
 extern "C" void
-mozilla_set_url_permission (const char *imgURL, gint type, gboolean allow)
+mozilla_set_url_permission(const char *imgURL, guint type, gboolean allow)
 {
 	nsCOMPtr<nsIPermissionManager> permissionManager =
-			do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
+			do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
-	permissionManager->Add (imgURL, allow ? PR_TRUE : PR_FALSE, type);
+#if MOZILLA_SNAPSHOT > 6
+	nsCOMPtr<nsIURI> url;
+	nsresult result = NS_NewURI(getter_AddRefs(url), nsDependentCString(imgURL));
+	if(NS_FAILED(result) || !url) return;
+
+	#if MOZILLA_SNAPSHOT > 9
+	permissionManager->Add(url, gPermType[type],
+	#else
+	permissionManager->Add(url, type,
+	#endif
+			       allow ?
+			       static_cast<PRUint32>(nsIPermissionManager::ALLOW_ACTION) :
+			       static_cast<PRUint32>(nsIPermissionManager::DENY_ACTION));
+#elif MOZILLA_SNAPSHOT > 2
+	permissionManager->Add(nsDependentCString(imgURL),
+			       allow ? PR_TRUE : PR_FALSE, type);
+#else
+	permissionManager->Add(imgURL, allow ? PR_TRUE : PR_FALSE, type);
+#endif
 }
 
 /**
@@ -1063,31 +1223,33 @@ mozilla_set_url_permission (const char *imgURL, gint type, gboolean allow)
  */
  
 extern "C" void
-mozilla_set_permission (GaleonEmbed *embed, gboolean permit, int type)
+mozilla_set_permission(GaleonEmbed *embed, gboolean allow, guint type)
 {
 	GALEON_WRAPPER(wrapper)
-	wrapper->SetSitePermission (permit ? PR_TRUE : PR_FALSE, type);
+	wrapper->SetSitePermission(allow ? PR_TRUE : PR_FALSE, type);
 }
 
 /**
  * mozilla_remove_cookies:
  * @gone: list of cookies which are to be removed
+ * @block: whether to block these cookies in future
  */
 extern "C" gboolean 
-mozilla_remove_cookies (GList *gone)
+mozilla_remove_cookies (GList *gone, gboolean block)
 {
 	nsresult result;
 	nsCOMPtr<nsICookieManager> cookieManager =
-			do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+			do_GetService(NS_COOKIEMANAGER_CONTRACTID);
 
 	for (GList *cookies = g_list_first(gone) ; cookies!=NULL ; 
 	     cookies = g_list_next(cookies))
 	{
 		Cookie *c = (Cookie *)cookies->data;
 
-		result = cookieManager->Remove (c->base.domain,c->name,
-					 	c->path,
-						PR_FALSE);
+		result = cookieManager->Remove(nsDependentCString(c->base.domain),
+					       nsDependentCString(c->name),
+					       nsDependentCString(c->path),
+					       block ? PR_TRUE : PR_FALSE);
 		if (NS_FAILED(result)) return FALSE;
 	};
 	return TRUE;
@@ -1099,17 +1261,25 @@ mozilla_remove_cookies (GList *gone)
  * @type: type of permissions ( cookies or images )
  */
 extern "C" gboolean 
-mozilla_remove_permissions (GList *gone, int type)
+mozilla_remove_permissions (GList *gone, guint type)
 {
 	nsresult result;
 	nsCOMPtr<nsIPermissionManager> permissionManager =
-			do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
+			do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
 	for (GList *permissions = g_list_first(gone) ; permissions != NULL;
 	     permissions = g_list_next(permissions))
 	{
 		BlockedHost *b = (BlockedHost *)permissions->data;
-		result = permissionManager->Remove (b->domain,type);
+#if MOZILLA_SNAPSHOT > 9
+		result = permissionManager->Remove(nsDependentCString(b->domain),
+						   gPermType[type]);
+#elif MOZILLA_SNAPSHOT > 2
+		result = permissionManager->Remove(nsDependentCString(b->domain),
+						   type);
+#else
+		result = permissionManager->Remove(b->domain,type);
+#endif
 		if (NS_FAILED(result)) return FALSE;
 	};
 	return TRUE;
@@ -1134,6 +1304,61 @@ mozilla_copy_session_history (GaleonEmbed *embed, GaleonEmbed *dest)
 /**
  * mozilla_get_charsets: get a list of charset supported by mozilla
  */
+#if MOZILLA_SNAPSHOT > 9
+extern "C" void
+mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
+{
+	nsresult rv;
+	PRUint32 translated_cscount = get_translated_cscount();
+
+	nsCOMPtr<nsICharsetConverterManager> ccm =
+		do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
+	if (!NS_SUCCEEDED(rv)) return;
+
+	nsCOMPtr <nsIUTF8StringEnumerator> e;
+	rv = ccm->GetDecoderList(getter_AddRefs(e));
+	if (!NS_SUCCEEDED(rv)) return;
+
+	*charsets = g_hash_table_new (g_str_hash, g_str_equal);
+
+	PRBool more;
+	for (rv = e->HasMore(&more); more; rv = e->HasMore(&more))
+	{
+		if (NS_FAILED(rv)) return;
+
+		nsCAutoString charset;
+		rv = e->GetNext(charset);
+		if (NS_FAILED(rv)) return;
+
+                nsAutoString title;                                             
+                rv = ccm->GetCharsetTitle(charset.get(), title);               
+		if (NS_FAILED(rv)) continue;
+
+		char *charset_title_str = convert_ns_string_to_c_string(title);
+
+		for (PRUint32 j = 0; j < translated_cscount; j++)
+		{
+			if (g_strcasecmp (
+				charset.get(), 
+				charset_trans_array[j].charset_name) == 0)
+			{
+				g_free(charset_title_str);
+				charset_title_str = (char *) 
+					_(charset_trans_array[j].charset_title);
+				break;
+			}
+		}
+
+		/* fill the hash and the sorted list */
+		g_hash_table_insert (*charsets, charset_title_str,
+				     g_strdup(charset.get()));
+		*sorted_charset_titles = g_list_insert_sorted (
+						(GList*)*sorted_charset_titles,
+						(gpointer)charset_title_str,
+						(GCompareFunc)g_strcasecmp); 
+	}
+}
+#else
 extern "C" void
 mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 {
@@ -1158,7 +1383,7 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 		nsCOMPtr<nsISupports> cssupports =
 					(dont_AddRef)(cs_list->ElementAt(i));
 		nsCOMPtr<nsIAtom> csatom ( do_QueryInterface(cssupports) );
-		nsString charset_ns = nsnull, charset_title_ns = nsnull;
+		nsAutoString charset_ns, charset_title_ns;
 
 		/* charset name */
 		rv = csatom->ToString(charset_ns);
@@ -1200,6 +1425,7 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 						(GCompareFunc)g_strcasecmp); 
 	}
 }
+#endif
 
 /**
  * mozilla_force_character_set: force the embed to use the specified
@@ -1228,16 +1454,9 @@ mozilla_clear_cache (unsigned long cache)
 
 /* This nonsense is needed to get the allocators right */
 static char *
-convert_ns_string_to_c_string (const nsString & ns_string)
+convert_ns_string_to_c_string (const nsAString & ns_string)
 {
-	char *c_string;
-	nsCString ns_c_string;
-
-	ns_c_string.AssignWithConversion (ns_string);
-
-	c_string = g_strdup (ns_c_string.get());
-
-	return c_string;
+	return g_strdup(NS_ConvertUCS2toUTF8(ns_string).get());
 }
 
 extern "C" gboolean
@@ -1245,7 +1464,7 @@ mozilla_get_favicon_location (GaleonEmbed *embed, gchar **url)
 {
 	GALEON_WRAPPER_IF_FAIL(wrapper, FALSE)
 	
-	nsCString transfer;
+	nsCAutoString transfer;
 	nsresult rv = wrapper->GetFaviconURL (transfer);
 	if(transfer.IsEmpty())
 		*url = NULL;
@@ -1265,7 +1484,7 @@ mozilla_list_passwords (PasswordType type)
 	nsresult result = NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIPasswordManager> passwordManager =
-			do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
+			do_GetService(NS_PASSWORDMANAGER_CONTRACTID);
 	nsCOMPtr<nsISimpleEnumerator> passwordEnumerator;
 	if (type == PASSWORD_PASSWORD)
 		result = passwordManager->GetEnumerator 
@@ -1283,24 +1502,24 @@ mozilla_list_passwords (PasswordType type)
 		nsCOMPtr<nsIPassword> nsPassword;
 		result = passwordEnumerator->GetNext 
 					(getter_AddRefs(nsPassword));
-		g_return_val_if_fail (NS_SUCCEEDED(result), NULL);
+		g_return_val_if_fail(NS_SUCCEEDED(result), NULL);
 
-		Password *p = g_new0 (Password, 1);
+		Password *p = g_new0(Password, 1);
 
-		nsXPIDLCString transfer;
-		nsPassword->GetHost (getter_Copies(transfer));
-		p->host = g_strdup (transfer.get());
+		nsCAutoString transfer;
+		nsPassword->GetHost(transfer);
+		p->host = g_strdup(transfer.get());
 
 		if (type == PASSWORD_PASSWORD)
 		{
-			nsXPIDLString unicodeName;
-			nsPassword->GetUser (getter_Copies(unicodeName));
-			p->username = mozilla_unicode_to_locale (unicodeName.get());
+			nsAutoString unicodeName;
+			nsPassword->GetUser(unicodeName);
+			p->username = mozilla_unicode_to_locale(unicodeName.get());
 		}
 		
-		passwords = g_list_prepend (passwords, p);
+		passwords = g_list_prepend(passwords, p);
 	}	
-	passwords = g_list_reverse (passwords);
+	passwords = g_list_reverse(passwords);
 	return passwords;
 }
 
@@ -1314,7 +1533,7 @@ mozilla_remove_passwords (GList *gone, PasswordType type)
 {
 	nsresult result = NS_ERROR_FAILURE;
 	nsCOMPtr<nsIPasswordManager> passwordManager =
-			do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
+			do_GetService(NS_PASSWORDMANAGER_CONTRACTID);
 
 	for (GList *passwords = g_list_first(gone) ; passwords!=NULL ; 
 	     passwords = g_list_next(passwords))
@@ -1324,13 +1543,16 @@ mozilla_remove_passwords (GList *gone, PasswordType type)
 		{
 			PRUnichar *unicodeName = mozilla_locale_to_unicode
 							(p->username);
-			result = passwordManager->RemoveUser (p->host,
-							      unicodeName);
-			g_free (unicodeName);
+
+			nsDependentString unicodeString(unicodeName);
+			result = passwordManager->RemoveUser(nsDependentCString(p->host),
+							     unicodeString);
+			g_free(unicodeName);
 		}
 		else if (type == PASSWORD_REJECT)
 		{
-			result = passwordManager->RemoveReject (p->host);
+			result = passwordManager->RemoveReject
+					(nsDependentCString(p->host));
 		};
 
 		if (NS_FAILED(result)) return FALSE;
@@ -1348,9 +1570,7 @@ mozilla_unicode_to_utf8 (const PRUnichar *uniStr)
 	PRInt32 sSize,dSize;
 	nsresult result;
 
-	const nsString str (uniStr);
-
-	sSize = str.Length ();
+	sSize = nsDependentString(uniStr).Length ();
 
 	nsCOMPtr<nsIUnicodeEncoder> 
 		unicodeEncoder = do_CreateInstance (UTF8_ENCODER_ID);
@@ -1385,7 +1605,11 @@ mozilla_unicode_to_locale (const PRUnichar *uniStr)
 {
 	PRInt32 sSize,dSize;
 	gchar *output;
+#if MOZILLA_SNAPSHOT > 9
+	nsCAutoString platformCharset;
+#else
 	nsAutoString platformCharset;
+#endif
 	nsresult rv;
 
 	/* sanity */
@@ -1395,33 +1619,42 @@ mozilla_unicode_to_locale (const PRUnichar *uniStr)
 	}
 
 	nsCOMPtr<nsIPlatformCharset> svc;
-	nsCOMPtr<nsICharsetConverterManager> ccm;
-	nsCOMPtr<nsIUnicodeEncoder> encoder;
-	const nsString str (uniStr);
-
 	svc = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
+
 	if (NS_SUCCEEDED(rv))
 	{
 		rv = svc->GetCharset(kPlatformCharsetSel_Menu,
 				     platformCharset);
 	}
+
+	nsCOMPtr<nsICharsetConverterManager> ccm;
 	if (NS_SUCCEEDED(rv))
 	{
 		ccm = do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID,
 				     &rv);
 	}
+
+	nsCOMPtr<nsIUnicodeEncoder> encoder;
 	if (NS_SUCCEEDED(rv))
 	{
+#if MOZILLA_SNAPSHOT > 9
+		rv = ccm->GetUnicodeEncoder(platformCharset.get(),
+#else
 		rv = ccm->GetUnicodeEncoder(&platformCharset,
+#endif
 					    getter_AddRefs(encoder));
 	}
+
+	const nsAString &str = nsDependentString(uniStr);
 	if (NS_SUCCEEDED(rv)) {
 		sSize = str.Length ();
-		encoder->GetMaxLength (str.get(), sSize, &dSize);
+		encoder->GetMaxLength (PromiseFlatString(str).get(),
+				       sSize, &dSize);
 		if (dSize)
 		{
 			output = g_new0 (gchar, dSize + 1);
-			encoder->Convert (str.get(), &sSize, output, &dSize);
+			encoder->Convert (PromiseFlatString(str).get(),
+					  &sSize, output, &dSize);
 			encoder->Finish (output, &dSize);
 			encoder->Reset ();
 			return output;
@@ -1441,7 +1674,11 @@ mozilla_unicode_to_locale (const PRUnichar *uniStr)
  */
 extern "C" gchar *
 mozilla_unicode_to_page_locale (const PRUnichar *uniStr,
+#if MOZILLA_SNAPSHOT > 9
+				nsCAutoString &platformCharset)
+#else
 				nsAutoString &platformCharset)
+#endif
 {
 	PRInt32 sSize,dSize;
 	gchar *output;
@@ -1454,23 +1691,30 @@ mozilla_unicode_to_page_locale (const PRUnichar *uniStr,
 	}
 
 	nsCOMPtr<nsICharsetConverterManager> ccm;
-	nsCOMPtr<nsIUnicodeEncoder> encoder;
-	const nsString str (uniStr);
-
 	ccm = do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID,
 			     &rv);
+
+	nsCOMPtr<nsIUnicodeEncoder> encoder;
 	if (NS_SUCCEEDED(rv))
 	{
+#if MOZILLA_SNAPSHOT > 9
+		rv = ccm->GetUnicodeEncoder(platformCharset.get(),
+#else
 		rv = ccm->GetUnicodeEncoder(&platformCharset,
+#endif
 					    getter_AddRefs(encoder));
 	}
+
+	const nsAString &str = nsDependentString(uniStr);
 	if (NS_SUCCEEDED(rv)) {
 		sSize = str.Length ();
-		encoder->GetMaxLength (str.get(), sSize, &dSize);
+		encoder->GetMaxLength (PromiseFlatString(str).get(),
+				       sSize, &dSize);
 		if (dSize)
 		{
 			output = g_new0 (gchar, dSize + 1);
-			encoder->Convert (str.get(), &sSize, output, &dSize);
+			encoder->Convert (PromiseFlatString(str).get(),
+					  &sSize, output, &dSize);
 			encoder->Finish (output, &dSize);
 			encoder->Reset ();
 			return output;
@@ -1520,7 +1764,11 @@ mozilla_locale_to_unicode (const gchar *locStr)
 {
 	PRInt32 sSize,dSize;
 	PRUnichar *uniStr;
+#if MOZILLA_SNAPSHOT > 9
+	nsCAutoString platformCharset;
+#else
 	nsAutoString platformCharset;
+#endif
 	nsresult rv;
 
 	/* sanity */
@@ -1546,7 +1794,11 @@ mozilla_locale_to_unicode (const gchar *locStr)
 	}
 	if (NS_SUCCEEDED(rv))
 	{
+#if MOZILLA_SNAPSHOT > 9
+		rv = ccm->GetUnicodeDecoder(platformCharset.get(),
+#else
 		rv = ccm->GetUnicodeDecoder(&platformCharset,
+#endif
 					    getter_AddRefs(decoder));
 	}
 	if (NS_SUCCEEDED(rv))
@@ -1577,7 +1829,11 @@ mozilla_locale_to_unicode (const gchar *locStr)
  */
 extern "C" PRUnichar *
 mozilla_page_locale_to_unicode (const gchar *locStr,
+#if MOZILLA_SNAPSHOT > 9
+				nsCAutoString &platformCharset)
+#else
 				nsAutoString &platformCharset)
+#endif
 {
 	PRInt32 sSize,dSize;
 	PRUnichar *uniStr;
@@ -1606,7 +1862,11 @@ mozilla_page_locale_to_unicode (const gchar *locStr,
 	}
 	if (NS_SUCCEEDED(rv))
 	{
+#if MOZILLA_SNAPSHOT > 9
+		rv = ccm->GetUnicodeDecoder(platformCharset.get(),
+#else
 		rv = ccm->GetUnicodeDecoder(&platformCharset,
+#endif
 					    getter_AddRefs(decoder));
 	}
 	if (NS_SUCCEEDED(rv))
@@ -1637,14 +1897,8 @@ mozilla_utf8_to_locale (const gchar* utfstr)
 	{
 		return NULL;
 	}
-	PRUnichar* unistr = mozilla_utf8_to_unicode(utfstr);
-	if (unistr == NULL)
-	{
-		return NULL;
-	}
-	gchar* locstr = mozilla_unicode_to_locale(unistr);
-	g_free(unistr);
-	return locstr;
+	
+	return mozilla_unicode_to_locale(NS_ConvertUTF8toUCS2(utfstr).get());
 }
 
 extern "C" gchar *
@@ -1659,18 +1913,31 @@ mozilla_locale_to_utf8 (const gchar* locstr)
 	{
 		return NULL;
 	}
-	gchar* utfstr = mozilla_unicode_to_utf8(unistr);
+
+	gchar* utfstr = g_strdup (NS_ConvertUCS2toUTF8(unistr).get());
 	g_free(unistr);
 	return utfstr;
 }
 
 /**
- * mozilla_find_gtk_parent: retrives gtkwidget pointer to parent
- * of domWindow. It will be the gtkmozembed that holds it.
+ * mozilla_find_gtk_parent: retrives gtkwindow that contains
+ * the domwWindow. This is retrived from the GaleonWindow parent.
  * @domWindow: nsIDOMWindow to find gtk parent of.
  */
 extern "C" GtkWidget *
 mozilla_find_gtk_parent (nsIDOMWindow *domWindow)
+{
+	GaleonWindow *parent = mozilla_find_galeon_parent(domWindow);
+	return parent ? parent->wmain : NULL;
+}
+
+/**
+ * mozilla_find_galeon_parent: retrives GaleonWindow that contains
+ * the domWindow.
+ * @domWindow: nsIDOMWindow to find galeon parent of.
+ */
+extern "C" GaleonWindow *
+mozilla_find_galeon_parent (nsIDOMWindow *domWindow)
 {
 	nsresult result;
 	GaleonWindow *gwindow;
@@ -1701,7 +1968,7 @@ mozilla_find_gtk_parent (nsIDOMWindow *domWindow)
 	/* The nautilus view has not a GaleonWindpw */
 	if (gwindow == NULL) return NULL;
 	
-	return gwindow->wmain;
+	return gwindow;
 }
 
 extern "C" gboolean
@@ -1773,7 +2040,7 @@ mozilla_delete_temp_file_on_exit (const char *filename)
 {
 	nsresult rv;
 	nsCOMPtr<nsILocalFile> localFile;
-	rv = NS_NewLocalFile (filename, PR_TRUE, getter_AddRefs(localFile));
+	rv = NS_NewNativeLocalFile(nsDependentCString(filename), PR_TRUE, getter_AddRefs(localFile)); 
 	if (NS_FAILED(rv)) return;
 
 	nsCOMPtr<nsIExternalHelperAppService> helperService =
@@ -1819,21 +2086,24 @@ mozilla_show_javascript_console (void)
 extern "C" GList *
 mozilla_get_alternate_stylesheets (GaleonEmbed *embed)
 {
-	gchar *defaultname = NULL;
-	gint found = 0;
-	GList *csslist = NULL;
 	nsresult result;
 	GALEON_WRAPPER_IF_FAIL(wrapper, NULL)
 	
 	nsCOMPtr<nsIDOMStyleSheetList> list;
 	result = wrapper->GetStyleSheets(getter_AddRefs(list));
 	if (NS_FAILED(result) || !list) return NULL;
-	
-	PRUint32 count, i; 
+
+	PRUint32 count(0); 
 	result = list->GetLength(&count);
 	if (NS_FAILED(result)) return NULL;
 
-	for (i = 0; i < count; i++)
+	//Sometimes Mozilla wants to return -1 as an unsigned value...
+	if ((count+1) == 0) count = 0;
+
+	gint found(0);
+	gchar *defaultname(NULL);
+	GList *csslist(NULL);
+	for (PRUint32 i = 0; i < count; i++)
 	{
 		nsCOMPtr<nsIDOMStyleSheet> item;
 		result = list->Item(i, getter_AddRefs(item));
@@ -1841,7 +2111,7 @@ mozilla_get_alternate_stylesheets (GaleonEmbed *embed)
 
 		found++;
 
-		nsString string;
+		nsAutoString string;
 		result = item->GetTitle(string);
 		if (NS_FAILED(result)) continue;
 
@@ -1909,13 +2179,16 @@ mozilla_set_alternate_stylesheet (GaleonEmbed *embed, AlternateStyleSheet *css)
 	result = list->GetLength(&count);
 	if (NS_FAILED(result)) return;
 
+        //Sometimes Mozilla wants to return -1 as an unsigned value...
+        if ((count+1) == 0) count = 0;
+
 	for (i = 0; i < count; i++)
 	{
 		nsCOMPtr<nsIDOMStyleSheet> item;
 		result = list->Item(i, getter_AddRefs(item));
 		if (NS_FAILED(result) || !item) continue;
 		
-		nsString string;
+		nsAutoString string;
 		result = item->GetTitle(string);
 		if (NS_FAILED(result)) continue;
 
@@ -1963,13 +2236,16 @@ mozilla_get_selected_stylesheet (GaleonEmbed *embed)
 	result = list->GetLength(&count);
 	if (NS_FAILED(result)) return NULL;
 
+        //Sometimes Mozilla wants to return -1 as an unsigned value...
+        if ((count+1) == 0) count = 0;
+
 	for (i = 0; i < count; i++)
 	{
 		nsCOMPtr<nsIDOMStyleSheet> item;
 		result = list->Item(i, getter_AddRefs(item));
 		if (NS_FAILED(result) || !item) continue;
 
-		nsString string;
+		nsAutoString string;
 		result = item->GetTitle(string);
 		if (NS_FAILED(result)) continue;
 
@@ -2017,7 +2293,7 @@ stylesheet_is_alternate (nsIDOMStyleSheet *item)
 	
 	if (link)
 	{
-		nsString str;
+		nsAutoString str;
 		link->GetRel(str);
 		gchar *tmp = convert_ns_string_to_c_string(str);
 		
@@ -2098,9 +2374,7 @@ mozilla_make_security_tooltip (gpointer req)
 	result = secInfo->GetShortSecurityDescription(getter_Copies(tooltip));
 	if (NS_FAILED (result) || !tooltip) return NULL;
 
-	nsString string = nsString(tooltip);
-	
-	return convert_ns_string_to_c_string (string);
+	return convert_ns_string_to_c_string (tooltip);
 }
 
 extern "C" gchar*
@@ -2141,7 +2415,7 @@ mozilla_get_real_url (GaleonEmbed *embed)
 {
 	GALEON_WRAPPER_IF_FAIL (wrapper, NULL)
 
-	nsCString url;
+	nsCAutoString url;
 	nsresult rv = wrapper->GetRealURL (url);
 
 	return (NS_SUCCEEDED (rv) && !url.IsEmpty()) ? g_strdup(url.get()) : NULL;
@@ -2193,6 +2467,14 @@ mozilla_scroll_right (GaleonEmbed *embed)
 	wrapper->ScrollRight ();
 }
 
+extern "C" void
+mozilla_scroll_by (GaleonEmbed *embed, gint x, gint y)
+{
+	GALEON_WRAPPER (wrapper);
+
+	wrapper->ScrollBy (x, y);
+}
+
 extern "C" gchar *
 mozilla_get_last_modified (GaleonEmbed *embed)
 {
@@ -2254,7 +2536,7 @@ mozilla_list_css (GaleonEmbed *embed)
 		result = list->Item(i, getter_AddRefs(item));
 		if (NS_FAILED(result) || !item) continue;
 
-		nsString string;
+		nsAutoString string;
 		result = item->GetHref(string);
 		if (NS_FAILED(result)) continue;
 
@@ -2315,14 +2597,14 @@ mozilla_get_link_text (gpointer rn)
 
 	while (node && depth > 0)
 	{
-		nsString name;
+		nsAutoString name;
 		node->GetNodeName(name);
 		if (name.EqualsWithConversion("#text", PR_TRUE))
 		{
 			nsCOMPtr<nsIDOMCharacterData> cdata = do_QueryInterface(node);
-			nsString ctext;
+			nsAutoString ctext;
 			cdata->GetData(ctext);
-			nsString s;
+			nsAutoString s;
 			s.AssignWithConversion(" ");
 			ltext += s;
 			ltext += ctext;
@@ -2385,11 +2667,11 @@ mozilla_uri_get_parent (const char *uri)
 	nsCOMPtr<nsIURL> aURL = do_QueryInterface(aURI, &rv);
 	if (NS_FAILED(rv) || !aURL) return NULL;
 
-	nsCString dirPath;
+	nsCAutoString dirPath;
 	rv = aURL->GetDirectory (dirPath);
 	if (NS_FAILED(rv) || !dirPath.Length()) return NULL;
 
-	nsCString filePath;
+	nsCAutoString filePath;
 	rv = aURL->GetFilePath (filePath);
 	if (NS_FAILED(rv) || !filePath.Length()) return NULL;
 
@@ -2408,12 +2690,12 @@ mozilla_uri_get_parent (const char *uri)
 	else
 	{
 		PRInt32 nextSlash = filePath.RFind("/",PR_FALSE,trailingSlash-1);
-		nsCString parentPath;
+		nsCAutoString parentPath;
 		filePath.Left(parentPath, nextSlash);
 		aURI->SetPath(parentPath);
 	}
 
-	nsCString spec;
+	nsCAutoString spec;
 	aURI->GetSpec(spec);
 
 	return !spec.IsEmpty() ? g_strdup(spec.get()) : NULL;
@@ -2431,3 +2713,14 @@ mozilla_uri_has_parent (const char *uri)
 	}
 }
 
+extern GSidebar *GlobalSidebar;
+
+extern "C" gboolean
+mozilla_show_sidebar (GaleonWindow *window)
+{
+	if(!GlobalSidebar)
+		GlobalSidebar = new GSidebar();
+
+	return NS_SUCCEEDED(GlobalSidebar->SetWindow(window)) ?
+	       TRUE : FALSE;
+}
