@@ -67,6 +67,8 @@ typedef struct
 	WrapperContextInfo ctx;
 	gint button;
 	gint modifier;
+	gint x_root;
+	gint y_root;
 	guint timestamp;
 } WrapperMouseEventInfo;
 
@@ -92,7 +94,8 @@ typedef enum
 	ACTION_FAVICON,
 	ACTION_FAVICON_EDITOR,
 	ACTION_SET_PIXMAP,
-	ACTION_ADDBOOKMARK
+	ACTION_ADDBOOKMARK,
+	ACTION_SILENT
 } DownloadAction;
 
 typedef struct
@@ -187,13 +190,24 @@ void mozilla_cut_selection (GaleonEmbed *embed);
 void mozilla_copy_selection (GaleonEmbed *embed);
 void mozilla_paste (GaleonEmbed *embed);
 
+/* Printing */
+gboolean mozilla_print (GaleonEmbed *embed, PrintInfo *info,
+			gboolean preview);
+gboolean mozilla_print_preview_close (GaleonEmbed *embed);
+gint mozilla_print_preview_num_pages (GaleonEmbed *embed);
+gboolean mozilla_print_preview_navigate (GaleonEmbed *embed, short navType,
+					 gint pageNum);
+
+/* WebNavigation */
+gboolean mozilla_reload (GaleonEmbed *embed);
+gboolean mozilla_copy_page (GaleonEmbed *dest, GaleonEmbed *source,
+			    guint aDisplayType);
+
 /* Utils */
 gboolean mozilla_find (GaleonEmbed *embed,
-		       WrapperSearchProperties *properties);
+		       WrapperSearchProperties *properties,
+		       gboolean interactive);
 void mozilla_delete_temp_file_on_exit (const char *filename);
-gboolean mozilla_reload (GaleonEmbed *embed);
-gboolean mozilla_print (GaleonEmbed *embed, PrintInfo *info,
-		    gboolean preview);
 void mozilla_clear_cache (unsigned long cache);
 gboolean mozilla_set_offline (GaleonEmbed *embed, gboolean offline);
 void mozilla_free (gpointer p);
@@ -216,6 +230,7 @@ void mozilla_register_js_listener (void);
 void mozilla_evaluate_js (GaleonEmbed *embed, char *script);
 char *mozilla_uri_get_parent (const char *uri);
 gboolean mozilla_uri_has_parent (const char *uri);
+gboolean mozilla_show_sidebar (GaleonWindow *window);
 
 /* Page information */
 gchar *mozilla_get_last_modified (GaleonEmbed *embed);
@@ -241,14 +256,14 @@ void mozilla_force_character_set (GaleonEmbed *embed,
 				  char *force_character_set);
 
 /* Cookies manager */
-void mozilla_set_url_permission (const gchar *imgURL, gint type,
+void mozilla_set_url_permission (const gchar *imgURL, guint type,
 				 gboolean allow);
 GList *mozilla_list_cookies (void);
-void mozilla_set_permission (GaleonEmbed *embed, gboolean permit, 
-				    int type);
-GList *mozilla_get_permissions (int type);
-gboolean mozilla_remove_cookies (GList *gone);
-gboolean mozilla_remove_permissions (GList *gone, int type);
+void mozilla_set_permission (GaleonEmbed *embed, gboolean allow, 
+			     guint type);
+GList *mozilla_get_permissions (guint type);
+gboolean mozilla_remove_cookies (GList *gone, gboolean block);
+gboolean mozilla_remove_permissions (GList *gone, guint type);
 
 /* Password Manager */
 GList *mozilla_list_passwords (PasswordType type);
@@ -271,6 +286,7 @@ void mozilla_scroll_up (GaleonEmbed *embed);
 void mozilla_scroll_down (GaleonEmbed *embed);
 void mozilla_scroll_left (GaleonEmbed *embed);
 void mozilla_scroll_right (GaleonEmbed *embed);
+void mozilla_scroll_by (GaleonEmbed *embed, gint x, gint y);
 
 #ifdef __cplusplus
 }
@@ -289,10 +305,19 @@ gchar *mozilla_unicode_to_locale (const PRUnichar *uniStr);
 PRUnichar *mozilla_utf8_to_unicode (const gchar *utfStr);
 PRUnichar *mozilla_locale_to_unicode (const gchar *locStr);
 GtkWidget *mozilla_find_gtk_parent (nsIDOMWindow *domWindow);
+GaleonWindow *mozilla_find_galeon_parent (nsIDOMWindow *domWindow);
+#if MOZILLA_SNAPSHOT > 9
+gchar *mozilla_unicode_to_page_locale (const PRUnichar *uniStr,
+				       nsCAutoString &platformCharset);
+PRUnichar *mozilla_page_locale_to_unicode (const gchar *locStr,
+					   nsCAutoString &platformCharset);
+#else
 gchar *mozilla_unicode_to_page_locale (const PRUnichar *uniStr,
 				       nsAutoString &platformCharset);
 PRUnichar *mozilla_page_locale_to_unicode (const gchar *locStr,
 					   nsAutoString &platformCharset);
+
+#endif
 }
 
 #endif
