@@ -41,12 +41,14 @@
 #include <config.h>
 #endif
 
+#include "nsString.h"
 #include "nsIMailtoUrl.h"
 #include "nsIURI.h"
-#include "nsIFileSpec.h"
+#include "nsCRT.h"
 #include "nsCOMPtr.h"
 #include "nsIPrompt.h"
 #include "nsIAuthPrompt.h"
+#include "nsIComponentManager.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 
