@@ -230,6 +230,8 @@ notifiers [] =
 	"network.cookie.warnAboutCookies"},
 	{ CONF_PERSISTENT_COOKIES_BEHAVIOR, generic_mozilla_int_notifier,
 	"network.cookie.cookieBehavior"},
+	{ CONF_PERSISTENT_COOKIE_LIFETIME, generic_mozilla_bool_notifier,
+	"network.cookie.lifetime.enabled"},
 	{ CONF_PERSISTENT_PASSWORDS_SAVE, generic_mozilla_bool_notifier,
 	"signon.rememberSignons"},
 	{ CONF_NETWORK_HTTP_VERSION, mozilla_http_version_notifier,
