@@ -20,6 +20,11 @@ void mozilla_prefs_set (void);
 void mozilla_prefs_set_default_charset (void);
 void mozilla_prefs_set_languages (void);
 void mozilla_prefs_set_fonts_defaults (void);
-void mozilla_prefs_set_fonts (LanguageFontInfo *info, int lang_enc);
+//void mozilla_prefs_set_fonts (LanguageFontInfo *info, int lang_enc);
+int mozilla_prefs_set_font (gchar *name, gchar *key, int lang_enc);
+
 void mozilla_prefs_get_fonts (LanguageFontInfo *info, int lang_enc);
 void mozilla_prefs_set_user_agent (void);
+void mozilla_prefs_set_font_size (gchar *key, int size, int lang_enc);
+
+void mozilla_prefs_set_proxy (void);
