@@ -398,6 +398,7 @@ PrefInfo lookup_table[] =
 	{ "statusbar_rewrite_check", CONF_FILTERING_STATUSBAR_REWRITE, PT_TOGGLEBUTTON, NULL, PG_AUTOAPPLY },
 	{ "proxy_keepalive_checkbutton", CONF_NETWORK_PROXY_KA, PT_TOGGLEBUTTON, NULL, PG_PROXY },
 	{ "warn_cookie_checkbutton", CONF_PERSISTENT_COOKIE_WARN, PT_TOGGLEBUTTON, NULL, PG_AUTOAPPLY },
+	{ "session_cookie_checkbutton", CONF_PERSISTENT_COOKIE_LIFETIME, PT_TOGGLEBUTTON, NULL, PG_AUTOAPPLY },
 	{ "passwords_save_checkbutton", CONF_PERSISTENT_PASSWORDS_SAVE, PT_TOGGLEBUTTON, NULL, PG_AUTOAPPLY },
 	{ "tabbed_insert_new_tabs_check", CONF_TABS_TABBED_INSERT_NEW_TABS, PT_TOGGLEBUTTON, NULL, PG_AUTOAPPLY },
 	{ "ssl2_checkbutton", CONF_SECURITY_SSL2, PT_TOGGLEBUTTON, NULL, PG_AUTOAPPLY },
@@ -1105,6 +1106,11 @@ void
 prefs_load (void)
 {
 	mime_db_load ();
+
+	/* Reflect gconf proxy prefs to mozilla, and explictly
+	   load autoconf_url if appropriate */
+
+	mozilla_prefs_set_proxy ();
 }
 
 /**
