@@ -370,6 +370,10 @@ window_create (guint32 chrome_mask, GaleonWindow *previous)
 	{
 		bookmarks_editor_show_dock (window);
 	}
+	if (eel_gconf_get_integer (CONF_WINDOWS_SHOW_DOCK) == 3)
+	{
+		mozilla_show_sidebar (window);
+	}
 	
 	/* return the completed GaleonWindow */
 	return window;
@@ -457,7 +461,7 @@ window_init_menu_data (GaleonWindow *window)
 	window->related_links_menuitem = go_menu_uiinfo[7].widget;
 
 	/* tools menu */
-	window->java_console = tools_menu_uiinfo[6].widget;
+	window->java_console = tools_menu_uiinfo[8].widget;
 
 	window->open_url_entry = NULL;
 
@@ -799,10 +803,11 @@ window_statusbar_set_security_icon (GaleonWindow *window, gboolean state,
 		gtk_pixmap_set (GTK_PIXMAP (window->security_icon),
 				pixmap->pixmap, pixmap->mask);
 	}
-	
-	gtk_tooltips_set_tip (window->bookmarks_tooltips,
-			      window->security_eventbox, tmp, NULL);
 
+	if (window->bookmarks_tooltips)
+		gtk_tooltips_set_tip (window->bookmarks_tooltips,
+				      window->security_eventbox, tmp, NULL);
+	
 	if (!tooltip) g_free (tmp);
 }
 
@@ -1812,6 +1817,29 @@ window_undock (GaleonWindow *window)
 		return;
 	}
 
+	/* Silly hack to ensure our mozilla sidebar's mozembed is
+	   removed from the dock before the dock is deconstructed.
+	   If this is not done, the mozembed get's tangled up with
+	   the rest of the structures in the window leading to
+	   Bad Things(tm) later when attempts are made to reference it.
+	   This is a single producer - single consumer problem
+	   where neither is re-entrant, so it is safe to employ
+	   the dock_type as a crude asynchronous communication
+	   channel. The signal is a kludge to avoid having to
+	   write a new signal type, which would require defining
+	   a new gtk class. I don't consider it worth it.
+	   When GSidebar recives the signal, it will remove the
+	   gtkmozembed and then change the dock_type. Once this
+	   is done, it's safe to continue the undocking. */
+
+	if (window->dock_type == DOCK_SIDEBAR)
+	{
+		gtk_signal_emit_by_name (GTK_OBJECT(window->docked),
+					 "debug-msg",
+					 "disconnect-mozembed");
+		while(window->dock_type == DOCK_SIDEBAR);
+	}
+
 	/* store width in pref */
 	if (window->docked_width_pref != NULL && 
 	    GTK_PANED (window->hpaned)->position_set)
@@ -1832,12 +1860,12 @@ window_undock (GaleonWindow *window)
 	gtk_widget_destroy (GTK_WIDGET (window->docked));
 	window->docked = NULL;
 
+	/* set dock type */
+	window->dock_type = DOCK_NONE;
+
 	/* update toolbar buttons */
 	window_update_bm_and_hist_buttons (window, FALSE, FALSE);
 
-	/* set dock type */
-	window->dock_type = DOCK_NONE;
-	
 	/* show the notebook again */
 	gtk_widget_show (GTK_WIDGET (window->notebook));
 }
