@@ -1012,7 +1012,6 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	    (event->state == 0 && event->keyval == GDK_BackSpace))
 	{
 		auto_completion_reset();
-		embed->location_is_modified = TRUE;
 		return TRUE;
 	}
 
@@ -1070,6 +1069,8 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 		return TRUE;
 	case GDK_Up:
 	case GDK_Down:
+	case GDK_Page_Up:
+	case GDK_Page_Down:
 		auto_completion_reset();
 		embed_grab_focus (window->active_embed);
 		return TRUE;
@@ -1138,7 +1139,6 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 			before_completion = NULL;
 		}
 		auto_completion_display (window, widget);
-		embed->location_is_modified = TRUE;
 		return TRUE;
 	}
 
@@ -1830,6 +1830,7 @@ window_history_button_released_cb (GtkButton *button, GdkEventButton *event,
 		window_undock (window);
 		GTK_TOGGLE_BUTTON (button)->active = TRUE;
 		break;
+	case DOCK_SIDEBAR:
 	case DOCK_BOOKMARKS:
 	case DOCK_NONE:
 		history_show_dock (window);
@@ -1865,6 +1866,7 @@ window_bookmarks_button_released_cb (GtkButton *button, GdkEventButton *event,
 		window_undock (window);
 		GTK_TOGGLE_BUTTON (button)->active = TRUE;
 		break;
+	case DOCK_SIDEBAR:
 	case DOCK_HISTORY:
 	case DOCK_NONE:
 		bookmarks_editor_show_dock (window);
@@ -2085,9 +2087,8 @@ window_notebook_switch_page_cb (GtkNotebook *notebook,
 	window_update_tab_controls (window);
 
 	/* update the statusbar */
-	if (!(old_embed && embed->secure == old_embed->secure))
-		window_statusbar_set_security_icon (window, embed->secure,
-						    embed->security_tooltip);
+        window_statusbar_set_security_icon (window, embed->secure,
+					    embed->security_tooltip);
 	window_statusbar_update_message (window);
 	window_statusbar_update_progress_bar (window);
 
@@ -2141,6 +2142,7 @@ window_notebook_get_page_at_location (GtkNotebook *notebook, gint x, gint y)
 	GtkNotebookPage *page;
 	GtkWidget *tab;
 	gint i, page_num = 0;
+        gboolean first_visible = TRUE;
 
 	/* get the relative position of the location, with regards to the
 	 * position of the notebook */
@@ -2156,10 +2158,17 @@ window_notebook_get_page_at_location (GtkNotebook *notebook, gint x, gint y)
 		page = l->data;
 		tab = page->tab_label;
 
+                /* avoid selecting the tabs that are not currently displayed */
+                if (!GTK_WIDGET_MAPPED(tab)) 
+                {
+                        continue;
+                }
+
 		/* make sure that our location is at least at the point
 		 * of the first tab */
-		if (i == 0)
+		if (first_visible)
 		{
+                        first_visible = FALSE;
 			if (x_rel < tab->allocation.x)
 				x_rel = tab->allocation.x;
 			if (y_rel < tab->allocation.y)
@@ -2178,6 +2187,7 @@ window_notebook_get_page_at_location (GtkNotebook *notebook, gint x, gint y)
 					page_num = i + 1;
 				else page_num = i;
 			}
+			else break;
 		}
 		else
 		{
@@ -2188,6 +2198,7 @@ window_notebook_get_page_at_location (GtkNotebook *notebook, gint x, gint y)
 					page_num = i + 1;
 				else page_num = i;
 			}
+			else break;
 		}
 	}
 
@@ -2223,6 +2234,12 @@ window_notebook_get_dest_page_at_location (GtkNotebook *notebook,
 		page = l->data;
 		tab = page->tab_label;
 
+                /* avoid selecting the tabs that are not currently displayed */
+                if (!GTK_WIDGET_MAPPED(tab)) 
+                {
+                        continue;
+                }
+
 		/* handle top/bottom and left/right tabs appropriately */
 		if (notebook->tab_pos == GTK_POS_TOP ||
 		    notebook->tab_pos == GTK_POS_BOTTOM)
@@ -2343,9 +2360,13 @@ window_notebook_button_press_cb (GtkWidget *widget, GdkEventButton *e,
 					              "button-press-event");
 			return TRUE;
 		}
-		
+
 		/* clone */
-		embed = window->active_embed;
+		embed = gtk_object_get_data
+			(GTK_OBJECT (((GtkNotebookPage *)(l->data))->tab_label),
+			 "GaleonEmbed");
+		return_val_if_not_embed (embed, FALSE);
+
 		tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED) ? 1 : 0;
 		shift_modifier = (e->state & GDK_SHIFT_MASK) ? 0 : 1;
 		new_embed = embed_create_after_embed (embed,
@@ -2464,6 +2485,8 @@ window_notebook_button_press_cb (GtkWidget *widget, GdkEventButton *e,
 			embed_close (embed);
 			break;
 		case 7:
+			/* set the dont_autosave flag, so we don't save the
+			 * session once for each of the embeds we close */
 			dont_autosave_session = TRUE;
 			for (l = g_list_last (window->embed_list); l;
 			     l = g_list_previous (l))
@@ -2473,6 +2496,7 @@ window_notebook_button_press_cb (GtkWidget *widget, GdkEventButton *e,
 					embed_close (e);
 			}
 			dont_autosave_session = FALSE;
+			/* do the save now */
 			session_autosave ();
 			break;
 		default:
